@@ -1,7 +1,7 @@
 /*=============================================================================
 crypto.c
   Copyright (C) W. Michael Putello <mike@flyn.org>, 2002
-  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2005
+  Copyright © Jan Engelhardt <jengelh [at] gmx de>, 2005 - 2006
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
@@ -35,33 +35,41 @@ crypto.c
 #else
 #    define EVP_MAX_BLOCK_LENGTH 0	/* FIXME: this is ugly, but needed */
 #endif
+#include "compiler.h"
 #include "crypto.h"
 #include "misc.h"
+#include "private.h"
 
+// Functions
 #ifdef HAVE_LIBCRYPTO
 static int hash_authtok(FILE *, const EVP_CIPHER *, const char *,
-  unsigned char *, unsigned char *);
+    unsigned char *, unsigned char *);
 static void sslerror(const char *);
 #endif
 
 //-----------------------------------------------------------------------------
 #ifdef HAVE_LIBCRYPTO
-/* human readable SSL error message */
-static void sslerror(const char *msg)
-{
+/*  sslerror
+    @msg:       Prefix to display
+
+    Print the human-readable form of the current SSL error.
+*/
+static void sslerror(const char *msg) {
 	unsigned long err = ERR_get_error();
-	if (err != 0) {
+	if(err != 0)
 		l0g(PMPREFIX "%s: %s", msg, ERR_error_string(err, NULL));
-	}
 }
 
-/* ============================ hash_authtok () ============================ */
-/* INPUT: fp, file containing encrypted stream; cipher; authtok, the key
- *        to unlock stream contained in fs
- * SIDE EFFECTS: hash, hash(authtok); iv, the initialization vector for fp;
- *               errors are logged
- * OUTPUT: if error 0 else 1
- */
+
+/*  hash_authtok
+    @fp:        stream containing encrypted stream
+    @cipher:    OpenSSL cipher struct
+    @authtok:   key to unlock stream
+    @hash:
+    @iv:        initialization vector for @fp
+
+    Returns zero on error or positive non-zero on success.
+*/
 static int hash_authtok(FILE *fp, const EVP_CIPHER *cipher,
  const char *authtok, unsigned char *hash, unsigned char *iv)
 {
@@ -85,36 +93,39 @@ static int hash_authtok(FILE *fp, const EVP_CIPHER *cipher,
 		return 0;
 	}
 	md = EVP_md5();
-        if(EVP_BytesToKey(cipher, md, salt, (const unsigned char *)authtok,
-         strlen(authtok), 1, hash, iv) <= 0) {
+        if(EVP_BytesToKey(cipher, md, salt,
+          signed_cast(const unsigned char *, authtok),
+          strlen(authtok), 1, hash, iv) <= 0) {
 		l0g(PMPREFIX "failed to hash system password\n");
 		return 0;
 	}
 
 	return 1;
 }
-#endif				/* HAVE_LIBCRYPTO */
 
-/* ============================ decrypted_key () =========================== */
-/* INPUT: fs_key_path, the path to an encrypted file (efsk); fs_key_cipher,
- *        the cipher used to encrypt the file; authtok, the key to unlock the
- *        file at fs_key_path
- * SIDE EFFECTS: pt_fs_key points to the decrypted data from the file at
- *               fs_key_path (fsk); pt_fs_key_len is the length of pt_fs_key;
- *               errors are logged
- * OUTPUT: if error 0 else 1
- * NOTE: pt_fs_key must point to a memory block large enough to hold fsk
- *       (MAX_PAR + EVP_MAX_BLOCK_LENGTH -- length of ct_fs_key + one block)
- *       efsk = encrypted filesystem key (stored in filesystem)
- *       fsk = filesystem key (D(efsk))
- *       pt_fs_key will contain binary data; don't use strlen, strcpy, etc.
- *       pt_fs_key may contain trailing garbage; use pt_fs_key_len
- */
+
+/*  decrypted_key
+    @pt_fs_key:         filesystem key
+    @pt_fs_key_len:     filesystem key length
+    @fs_key_path:       path to an encrypted file (EFSK)
+    @fs_key_cipher:     the cipher used to encrypt the file
+    @authtok:           key to unlock the file at @fs_key_path
+
+    Returns zero on error or positive non-zero on success.
+
+    NOTES:
+     *  @pt_fs_key must point to a memory block large enough to hold fsk
+        (MAX_PAR + EVP_MAX_BLOCK_LENGTH -- length of @ct_fs_key + one block)
+     *  efsk = encrypted filesystem key (stored in filesystem)
+     *  fsk = filesystem key (D(efsk))
+     *  @pt_fs_key will contain binary data; do not use strlen(), strcpy(), etc.
+     *  @pt_fs_key may contain trailing garbage; use @pt_fs_key_len
+
+    FIXME: this function may need to be broken up and made more readable.
+*/
 int decrypted_key(unsigned char *pt_fs_key, size_t *pt_fs_key_len,
  const char *fs_key_path, const char *fs_key_cipher, const char *authtok)
 {
-/* FIXME: this function may need to be broken up and made more readable */
-#ifdef HAVE_LIBCRYPTO
 	int ret = 1;
 	int segment_len;
 	unsigned char ct_fs_key[MAX_PAR];	/* encrypted filesystem key. */
@@ -146,8 +157,7 @@ int decrypted_key(unsigned char *pt_fs_key, size_t *pt_fs_key_len,
 		ret = 0;
 		goto _return;
 	}
-	if (hash_authtok(fs_key_fp, cipher, authtok, hashed_authtok, iv) ==
-	    0) {
+        if(hash_authtok(fs_key_fp, cipher, authtok, hashed_authtok, iv) == 0) {
 		ret = 0;
 		goto _return;
 	}
@@ -156,24 +166,22 @@ int decrypted_key(unsigned char *pt_fs_key, size_t *pt_fs_key_len,
 		ret = 0;
 		goto _return;
 	}
-	if (EVP_DecryptInit_ex(&ctx, cipher, NULL, hashed_authtok, iv) ==
-	    0) {
+        if(EVP_DecryptInit_ex(&ctx, cipher, NULL, hashed_authtok, iv) == 0) {
 		sslerror("failed to initialize decryption code");
 		ret = 0;
 		goto _return;
 	}
 	/* assumes plaintexts is always <= ciphertext + EVP_MAX_BLOCK_LEN in length
 	 * OpenSSL's documentation seems to promise this */
-	if (EVP_DecryptUpdate
-	    (&ctx, pt_fs_key, &segment_len, ct_fs_key,
-	     ct_fs_key_len) == 0) {
+        if(EVP_DecryptUpdate(&ctx, pt_fs_key, &segment_len, ct_fs_key,
+          ct_fs_key_len) == 0) {
 		sslerror("failed to decrypt key");
 		ret = 0;
 		goto _return;
 	}
 	*pt_fs_key_len = segment_len;
-	if (EVP_DecryptFinal_ex
-	    (&ctx, &pt_fs_key[*pt_fs_key_len], &segment_len) == 0) {
+        if(EVP_DecryptFinal_ex(&ctx, &pt_fs_key[*pt_fs_key_len],
+          &segment_len) == 0) {
 		sslerror
 		    ("bad pad on end of encrypted file (wrong algorithm or key size?)");
 		ret = 0;
@@ -193,12 +201,20 @@ int decrypted_key(unsigned char *pt_fs_key, size_t *pt_fs_key_len,
 
 	ERR_free_strings();
 	/* pt_fs_key_len is unsigned */
-	assert(ret == 0
-	       || *pt_fs_key_len <= MAX_PAR + EVP_MAX_BLOCK_LENGTH);
+        assert(ret == 0 || *pt_fs_key_len <= MAX_PAR + EVP_MAX_BLOCK_LENGTH);
 
 	return ret;
-#else
-	l0g(PMPREFIX "encrypted filesystem key not supported: no openssl\n");
-	return 0;
-#endif				/* HAVE_LIBCRYPTO */
 }
+
+#else // HAVE_LIBCRYPTO
+
+int decrypted_key(unsigned char *pt_fs_key, size_t *pt_fs_key_len,
+ const char *fs_key_path, const char *fs_key_cipher, const char *authtok)
+{
+    l0g(PMPREFIX "encrypted filesystem key not supported: no openssl\n");
+    return 0;
+}
+
+#endif // HAVE_LIBCRYPTO
+
+//=============================================================================
