@@ -1,7 +1,7 @@
 /*=============================================================================
 dotconf.c
   Copyright (C) Lukas Schroeder <lukas@azzit.de>, 1999 - 2002
-  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2005
+  Copyright © Jan Engelhardt <jengelh [at] gmx de>, 2005 - 2006
   and others
 
   This program is free software; you can redistribute it and/or modify
@@ -23,10 +23,6 @@ dotconf.c
 =============================================================================*/
 /* -- dotconf.c - this code is responsible for the input, parsing and dispatching of options  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 /* Added by Stephen W. Boyer <sboyer@caldera.com>
  * for wildcard support in Include file paths
  */
@@ -49,7 +45,9 @@ dotconf.c
 #ifdef HAVE_SYSLOG
 #    include <syslog.h>
 #endif
+#include "compiler.h"
 #include "dotconf.h"
+#include "xstdlib.h"
 
 // some buffersize definitions
 #define CFG_BUFSIZE             4096    // max length of one line */
@@ -70,7 +68,7 @@ dotconf.c
 )
 
 // for convenience of terminating the dotconf_options list
-#define LAST_CONTEXT_OPTION     {"", 0, NULL, NULL, 0}
+#define LAST_CONTEXT_OPTION     {""}
 
 // some flags that change the runtime behaviour of dotconf
 #define NONE                   0
@@ -123,46 +121,40 @@ enum callback_types
 
 typedef enum callback_types callback_types;
 
-static const configoption_t *get_argname_fallback(const configoption_t *);
+// Functions
+static const struct configoption *get_argname_fallback(const struct configoption *);
 static void copy_word(char **, char **, int, char);
 static DOTCONF_CB(dotconf_cb_include);          // internal 'Include'
 static DOTCONF_CB(dotconf_cb_includepath);      // internal 'IncludePath'
 static int dotconf_continue_line(char *, size_t);
-static int dotconf_find_wild_card(const char *, char *, char **, char **,
-    const char **);
-static void dotconf_free_command(command_t *);
-static char *dotconf_get_here_document(configfile_t *, const char *);
-static int dotconf_get_next_line(char *, size_t, configfile_t *);
-static const char *dotconf_handle_command(configfile_t *, char *);
-static int dotconf_handle_star(const command_t *, const char *, const char *,
-    const char *);
-static int dotconf_handle_question_mark(const command_t *, const char *,
-    const char *, const char *);
-static int dotconf_handle_wild_card(const command_t *, char, const char *,
-    const char *, const char *);
-static const char *dotconf_invoke_command(const configfile_t *,
-    const command_t *);
+static int dotconf_find_wild_card(const char *, char *, char **, char **, const char **);
+static void dotconf_free_command(struct command *);
+static char *dotconf_get_here_document(struct configfile *, const char *);
+static int dotconf_get_next_line(char *, size_t, struct configfile *);
+static const char *dotconf_handle_command(struct configfile *, char *);
+static int dotconf_handle_star(const struct command *, const char *, const char *, const char *);
+static int dotconf_handle_question_mark(const struct command *, const char *, const char *, const char *);
+static int dotconf_handle_wild_card(const struct command *, char, const char *, const char *, const char *);
+static const char *dotconf_invoke_command(const struct configfile *, const struct command *);
 static int dotconf_is_wild_card(char);
-static int dotconf_question_mark_match(const char *, const char *,
-    const char *);
-static char *dotconf_read_arg(const configfile_t *, char **);
-static void dotconf_register_options(configfile_t *, const configoption_t *);
-static void dotconf_set_command(configfile_t *, const configoption_t *,
-    char *, command_t *);
+static int dotconf_question_mark_match(const char *, const char *, const char *);
+static char *dotconf_read_arg(const struct configfile *, char **);
+static void dotconf_register_options(struct configfile *, const struct configoption *);
+static void dotconf_set_command(struct configfile *, const struct configoption *, char *, struct command *);
 static int dotconf_star_match(const char *, const char *, const char *);
 static int dotconf_strcmp_from_back(const char *, const char *);
-static char *dotconf_substitute_env(const configfile_t *, char *);
-static int dotconf_warning(const configfile_t *, int, unsigned long,
-    const char *, ...);
+static char *dotconf_substitute_env(const struct configfile *, char *);
+static int dotconf_warning(const struct configfile *, int, unsigned long, const char *, ...);
 static void dotconf_wild_card_cleanup(char *, char *);
 static void skip_whitespace(char **, int, char);
 static inline long MIN(long, long);
 
-static configoption_t dotconf_options[] =
+// Variables
+static struct configoption dotconf_options[] =
 {
 	{ "Include", ARG_STR, dotconf_cb_include, NULL, CTX_ALL },
 	{ "IncludePath", ARG_STR, dotconf_cb_includepath, NULL, CTX_ALL },
-	LAST_CONTEXT_OPTION
+	LAST_CONTEXT_OPTION,
 };
 
 static char name[CFG_MAX_OPTION + 1]; // option name
@@ -188,7 +180,8 @@ static void copy_word(char **dest, char **src, int max, char term)
 	*dest = cp2;
 }
 
-static const configoption_t *get_argname_fallback(const configoption_t *options)
+static const struct configoption *
+get_argname_fallback(const struct configoption *options)
 {
 	int i;
 
@@ -198,7 +191,7 @@ static const configoption_t *get_argname_fallback(const configoption_t *options)
 	return NULL;
 }
 
-static char *dotconf_substitute_env(const configfile_t *configfile,
+static char *dotconf_substitute_env(const struct configfile *configfile,
  char *str)
 {
         // {{ editor lineup
@@ -269,10 +262,10 @@ static char *dotconf_substitute_env(const configfile_t *configfile,
 	*cp2 = '\0';				/* terminate buffer */
 
 	free(str);
-	return strdup(tmp_value);
+	return xstrdup(tmp_value);
 }
 
-static int dotconf_warning(const configfile_t *configfile, int type,
+static int dotconf_warning(const struct configfile *configfile, int type,
  unsigned long errnum, const char *fmt, ...)
 {
 	va_list args;
@@ -296,23 +289,18 @@ static int dotconf_warning(const configfile_t *configfile, int type,
 	return retval;
 }
 
-static void dotconf_register_options(configfile_t *configfile,
- const configoption_t * options)
+static void dotconf_register_options(struct configfile *configfile,
+ const struct configoption *options)
 {
 	int num = configfile->config_option_count;
 
 #define GROW_BY   10
-
 	/* resize memoryblock for options blockwise */
 	if (configfile->config_options == NULL)
 		configfile->config_options = malloc(sizeof(void *) * (GROW_BY + 1));
-	else
-	{
-		if((num % GROW_BY) == 0)
-			configfile->config_options = realloc(configfile->config_options,
-											 sizeof(void *) * (num + GROW_BY + 1));
-	}
-
+        else if((num % GROW_BY) == 0)
+	    configfile->config_options = realloc(configfile->config_options,
+                sizeof(void *) * (num + GROW_BY + 1));
 #undef GROW_BY
 
 	/* append new options */
@@ -342,7 +330,7 @@ static int dotconf_continue_line(char *buffer, size_t length) {
 }
 
 static int dotconf_get_next_line(char *buffer, size_t bufsize,
- configfile_t *configfile)
+ struct configfile *configfile)
 {
 	char *cp1, *cp2;
 	char buf2[CFG_BUFSIZE];
@@ -377,7 +365,7 @@ static int dotconf_get_next_line(char *buffer, size_t bufsize,
 	return 0;
 }
 
-static char *dotconf_get_here_document(configfile_t *configfile,
+static char *dotconf_get_here_document(struct configfile *configfile,
  const char *delimit)
 {
 	/* it's a here-document: yeah, what a cool feature ;) */
@@ -405,8 +393,7 @@ static char *dotconf_get_here_document(configfile_t *configfile,
 	 * allocate a buffer of filesize bytes; should be enough to
 	 * prevent buffer overflows
 	 */
-	here_doc = malloc(configfile->size);	/* allocate buffer memory */
-	memset(here_doc, 0, configfile->size);
+	here_doc = xzalloc(configfile->size);	/* allocate buffer memory */
 
 	here_string = 1;
 	limit_len = snprintf(here_limit, 9, "%s", delimit);
@@ -426,13 +413,15 @@ static char *dotconf_get_here_document(configfile_t *configfile,
 	return realloc(here_doc, offset);
 }
 
-static const char *dotconf_invoke_command(const configfile_t *configfile,
- const command_t *cmd)
+static const char *dotconf_invoke_command(const struct configfile *configfile,
+ const struct command *cmd)
 {
 	return cmd->option->callback(cmd, configfile->context);
 }
 
-static char *dotconf_read_arg(const configfile_t *configfile, char **line) {
+static char *dotconf_read_arg(const struct configfile *configfile,
+ char **line)
+{
 	int sq = 0, dq = 0;							/* single quote, double quote */
 	int done = 0;
 	char *cp1 = *line;
@@ -488,7 +477,7 @@ static char *dotconf_read_arg(const configfile_t *configfile, char **line) {
 		}
 		/* unquoted, unescaped comment-hash ; break out, unless NO_INLINE_COMMENTS is set */
 		else if (*cp1 == '#' && !dq && !sq && !(configfile->flags & NO_INLINE_COMMENTS)) {
-			/* 
+			/*
 			 * NOTE: 1.0.8a got the NO_INLINE_COMMENTS feature wrong: it
              * skipped every argument starting with a #, instead of simply eating it!
 			 */
@@ -513,27 +502,27 @@ static char *dotconf_read_arg(const configfile_t *configfile, char **line) {
 		BOTH! will be substituted, which is somewhat wrong, ain't it ?? :-(
 	*/
 	if(configfile->flags & DONT_SUBSTITUTE)
-		return (buf[0] != '\0') ? strdup(buf) : NULL;
- 	return (buf[0] != '\0') ? dotconf_substitute_env(configfile, strdup(buf)) : NULL;
+		return (buf[0] != '\0') ? xstrdup(buf) : NULL;
+ 	return (buf[0] != '\0') ? dotconf_substitute_env(configfile, xstrdup(buf)) : NULL;
 }
 
-static void dotconf_set_command(configfile_t *configfile,
- const configoption_t *option, char *args, command_t *cmd)
+static void dotconf_set_command(struct configfile *configfile,
+ const struct configoption *option, char *args, struct command *cmd)
 {
 	char *eob = args + strlen(args);
 
-	/* fill in the command_t structure with values we already know */
+	/* fill in the command structure with values we already know */
 	cmd->name = (option->type == ARG_NAME) ? name : option->name;
-	cmd->option = (configoption_t *)option;
+	cmd->option = const_cast(struct configoption *, option);
 	cmd->context = configfile->context;
 	cmd->configfile = configfile;
-	cmd->data.list = (char **)calloc(CFG_VALUES, sizeof(char *));
+	cmd->data.list = xzalloc(CFG_VALUES * sizeof(char *));
 	cmd->data.str = NULL;
 
 	if (option->type == ARG_RAW) {
 		/* if it is an ARG_RAW type, save some time and call the
 		   callback now */
-		cmd->data.str = strdup(args);
+		cmd->data.str = xstrdup(args);
 	}
 	else if (option->type == ARG_STR) {
 		char *cp = args;
@@ -553,15 +542,15 @@ static void dotconf_set_command(configfile_t *configfile,
 		skip_whitespace(&args, eob - args, '\0');
 
 		cmd->arg_count = 0;
-		while ( cmd->arg_count < (CFG_VALUES - 1)
-				&& (cmd->data.list[cmd->arg_count] = dotconf_read_arg(configfile, &args)) != NULL) {
+                while(cmd->arg_count < CFG_VALUES - 1 &&
+                  (cmd->data.list[cmd->arg_count] =
+                  dotconf_read_arg(configfile, &args)) != NULL)
 			cmd->arg_count++;
-		}
 
 		skip_whitespace(&args, eob - args, '\0');
 
 		if(cmd->arg_count > 0 && cmd->data.list[cmd->arg_count-1] != NULL && *args != '\0')
-			cmd->data.list[cmd->arg_count++] = strdup(args);
+			cmd->data.list[cmd->arg_count++] = xstrdup(args);
 
 		/* has an option entry been found before or do we have to use a fallback? */
 		if((option->name != NULL && option->name[0] > ' ') || option->type == ARG_NAME) {
@@ -597,7 +586,7 @@ static void dotconf_set_command(configfile_t *configfile,
 						return;
 					}
 
-					cmd->data.str = strdup(cmd->data.list[0]);
+					cmd->data.str = xstrdup(cmd->data.list[0]);
 					break;
 				case ARG_NAME:	/* fall through */
 				case ARG_LIST:
@@ -610,7 +599,7 @@ static void dotconf_set_command(configfile_t *configfile,
 	}
 }
 
-static void dotconf_free_command(command_t *command) {
+static void dotconf_free_command(struct command *command) {
 	int i;
 
 	free(command->data.str);
@@ -619,16 +608,16 @@ static void dotconf_free_command(command_t *command) {
 	free(command->data.list);
 }
 
-static const char *dotconf_handle_command(configfile_t *configfile,
+static const char *dotconf_handle_command(struct configfile *configfile,
  char *buffer)
 {
-	char *cp1; 
+	char *cp1;
 	char *cp2;
 	/* generic char pointer      */
 	char *eob;									/* end of buffer; end of string  */
 	const char *error = NULL;         /* error message we'll return */
 	const char *context_error = NULL; /* error message returned by contextchecker */
-	command_t command;							/* command structure */
+	struct command command;							/* command structure */
 	int mod = 0;
 	int next_opt_idx = 0;
 
@@ -653,7 +642,7 @@ static const char *dotconf_handle_command(configfile_t *configfile,
 	copy_word(&cp2, &cp1, MIN(eob - cp1, CFG_MAX_OPTION), '\0');
 
 	while (1) {
-		const configoption_t *option;
+		const struct configoption *option;
 		int done = 0;
 		int opt_idx = 0;
 
@@ -661,7 +650,7 @@ static const char *dotconf_handle_command(configfile_t *configfile,
 			for(opt_idx = next_opt_idx; configfile->config_options[mod][opt_idx].name[0] != '\0'; opt_idx++) {
 				if(configfile->cmp_func(name, configfile->config_options[mod][opt_idx].name, CFG_MAX_OPTION) == 0) {
 					/* TODO: this could be flagged: option overwriting by modules */
-					option = (configoption_t *) &configfile->config_options[mod][opt_idx];
+					option = &configfile->config_options[mod][opt_idx];
 					done = 1;
 					break;		/* found one; break out */
 				}
@@ -670,7 +659,7 @@ static const char *dotconf_handle_command(configfile_t *configfile,
 
 		if(option == NULL)
 			option = get_argname_fallback(configfile->config_options[1]);
-		
+
 		if(option == NULL || option->callback == NULL) {
 			if(error != NULL)
 				return error;
@@ -687,45 +676,38 @@ static const char *dotconf_handle_command(configfile_t *configfile,
 
 		if(context_error == NULL)
 			error = dotconf_invoke_command(configfile, &command);
-		else {
-			if(error == NULL) {
-				/* avoid returning another error then the first. This makes it easier to
-                   reproduce problems. */
-				error = context_error;
-			}
-		}
+                else if(error == NULL)
+                    /* Avoid returning another error then the first. This makes
+                    it easier to reproduce problems. */
+                    error = context_error;
 
 		dotconf_free_command(&command);
 
-		if(context_error == NULL || !(configfile->flags & DUPLICATE_OPTION_NAMES)) {
+		if(context_error == NULL || !(configfile->flags & DUPLICATE_OPTION_NAMES))
 			/* don't try more, just quit now. */
 			break;
-		}
 	}
 
 	return error;
 }
 
-int dotconf_command_loop(configfile_t *configfile)
-{
+int dotconf_command_loop(struct configfile *configfile) {
 	/* ------ returns: 0 for failure -- !0 for success ------------------------------------------ */
 	char buffer[CFG_BUFSIZE];
 
 	while(!(dotconf_get_next_line(buffer, sizeof(buffer), configfile))) {
 		const char *error = dotconf_handle_command(configfile, buffer);
 		if ( error != NULL )
-		{
 			if ( dotconf_warning(configfile, DCLOG_ERR, 0, error) )
 				return 0;
-		}
 	}
 	return 1;
 }
 
-configfile_t *dotconf_create(const char *fname, const configoption_t *options,
-                             context_t *context, unsigned long flags)
+struct configfile *dotconf_create(const char *fname,
+ const struct configoption *options, context_t *context, unsigned long flags)
 {
-	configfile_t *new = NULL;
+	struct configfile *new = NULL;
 	char *dc_env;
 
 	if (access(fname, R_OK))
@@ -734,7 +716,7 @@ configfile_t *dotconf_create(const char *fname, const configoption_t *options,
 		return NULL;
 	}
 
-	new = calloc(1, sizeof(configfile_t));
+	new = xzalloc(sizeof(struct configfile));
 	if((new->stream = fopen(fname, "r")) == NULL) {
 		fprintf(stderr, "Error opening configuration file '%s'\n", fname);
 		free(new);
@@ -742,7 +724,7 @@ configfile_t *dotconf_create(const char *fname, const configoption_t *options,
 	}
 
 	new->flags = flags;
-	new->filename = strdup(fname);
+	new->filename = xstrdup(fname);
 
 	new->includepath = malloc(CFG_MAX_FILENAME);
 	new->includepath[0] = '\0';
@@ -764,7 +746,7 @@ configfile_t *dotconf_create(const char *fname, const configoption_t *options,
 	return new;
 }
 
-void dotconf_cleanup(configfile_t *configfile) {
+void dotconf_cleanup(struct configfile *configfile) {
     fclose(configfile->stream);
     free(configfile->filename);
     free(configfile->config_options);
@@ -792,7 +774,7 @@ static int dotconf_is_wild_card(char value) {
 }
 
 /* ------ internal utility function that calls the appropriate routine for the wildcard passed in -- */
-static int dotconf_handle_wild_card(const command_t *cmd, char wild_card,
+static int dotconf_handle_wild_card(const struct command *cmd, char wild_card,
  const char *path, const char *pre, const char *ext)
 {
 	int retval = 0;
@@ -874,7 +856,7 @@ static int dotconf_find_wild_card(const char *filename, char *wildcard,
 
 				*ext = filename + prefix_len;
 				*wildcard = **ext;
-				(*ext)++;
+				++*ext;
 
 				retval = prefix_len;
 
@@ -915,22 +897,13 @@ static int dotconf_question_mark_match(const char *dir_name, const char *pre,
 	int ext_len = strlen(ext);
 	int w_card_check = strcspn(ext,WILDCARDS);
 
-	if(w_card_check < ext_len && strncmp(dir_name, pre, pre_len) == 0 &&
-	    strcmp(dir_name, ".") != 0 && strcmp(dir_name, "..") != 0)
-	{
-		retval = 1;    /* Another wildcard found */
+    if(w_card_check < ext_len && strncmp(dir_name, pre, pre_len) == 0 &&
+      strcmp(dir_name, ".") != 0 && strcmp(dir_name, "..") != 0)
+            retval = 1; /* Another wildcard found */
 
-	} else {
-
-		if(dir_name_len >= pre_len &&
-			 strncmp(dir_name, pre, pre_len) == 0 &&
-			 strcmp(dir_name, ".") != 0 &&
-			 strcmp(dir_name, "..") != 0)
-		{
-			retval = 0; /* Matches no other wildcards */
-		}
-
-	}
+    else if(dir_name_len >= pre_len && strncmp(dir_name, pre, pre_len) == 0 &&
+      strcmp(dir_name, ".") != 0 && strcmp(dir_name, "..") != 0)
+            retval = 0; /* Matches no other wildcards */
 
 	return retval;
 }
@@ -945,33 +918,25 @@ static int dotconf_star_match(const char *dir_name, const char *pre,
 	int ext_len = strlen(ext);
 	int w_card_check = strcspn(ext,WILDCARDS);
 
-	if(w_card_check < ext_len && strncmp(dir_name, pre, pre_len) == 0 &&
-	    strcmp(dir_name, ".") != 0 && strcmp(dir_name, "..") != 0)
-	{
-		retval = 1;    /* Another wildcard found */
+    if(w_card_check < ext_len && strncmp(dir_name, pre, pre_len) == 0 &&
+      strcmp(dir_name, ".") != 0 && strcmp(dir_name, "..") != 0)
+            retval = 1; /* Another wildcard found */
 
-	} else {
-
-		if(dir_name_len >= ext_len + pre_len &&
-			dotconf_strcmp_from_back(dir_name, ext) == 0 &&
-			strncmp(dir_name, pre, pre_len) == 0 &&
-			strcmp(dir_name, ".") != 0 &&
-			strcmp(dir_name, "..") != 0)
-		{
-			retval = 0; /* Matches no other wildcards */
-		}
-
-	}
+    else if(dir_name_len >= ext_len + pre_len &&
+      dotconf_strcmp_from_back(dir_name, ext) == 0 &&
+      strncmp(dir_name, pre, pre_len) == 0 && strcmp(dir_name, ".") != 0 &&
+      strcmp(dir_name, "..") != 0)
+            retval = 0; /* Matches no other wildcards */
 
 	return retval;
 }
 
 /* ------ internal utility function that determins matches for filenames with   -- */
 /* ------ a '?' in name and calls the Internal Include function on that filename -- */
-static int dotconf_handle_question_mark(const command_t *cmd, const char *path,
- const char *pre, const char *ext)
+static int dotconf_handle_question_mark(const struct command *cmd,
+ const char *path, const char *pre, const char *ext)
 {
-	configfile_t *included;
+	struct configfile *included;
 	DIR *dh = NULL;
 	struct dirent *dirptr = NULL;
 	int i;
@@ -1004,24 +969,14 @@ static int dotconf_handle_question_mark(const command_t *cmd, const char *path,
 				if ( !alloced )
 				{
 					if((new_path = malloc(new_path_len)) == NULL)
-					{
 						return -1;
-					}
 
 					alloced = new_path_len;
 
-				} else {
-
-						if ( new_path_len > alloced )
-						{
-							if ( realloc(new_path,new_path_len) == NULL )
-							{
-								free(new_path);
-								return -1;
-							}
-
-						}
-
+                                } else if(new_path_len > alloced &&
+                                  realloc(new_path,new_path_len) == NULL) {
+                                        free(new_path);
+                                        return -1;
 				}
 
 				if (match_state == 1)
@@ -1033,15 +988,10 @@ static int dotconf_handle_question_mark(const command_t *cmd, const char *path,
 					sprintf(new_path,"%s%s%s",path,new_pre,ext);
 
 					if (strcmp(new_path,already_matched) == 0)
-					{
 						continue; /* Already searched this expression */
-
-					} else {
-
+					else
 						strcpy(already_matched,new_path);
 
-					}
-
 					if (dotconf_find_wild_card(new_path,&wc,&wc_path,&wc_pre,&wc_ext) >= 0)
 					{
 						if ( dotconf_handle_wild_card(cmd,wc,wc_path,wc_pre,wc_ext) < 0)
@@ -1096,10 +1046,10 @@ static int dotconf_handle_question_mark(const command_t *cmd, const char *path,
 
 /* ------ internal utility function that determins matches for filenames with   --- */
 /* ------ a '*' in name and calls the Internal Include function on that filename -- */
-static int dotconf_handle_star(const command_t *cmd, const char *path,
+static int dotconf_handle_star(const struct command *cmd, const char *path,
  const char *pre, const char *ext)
 {
-	configfile_t *included;
+	struct configfile *included;
 	DIR *dh = NULL;
 	struct dirent *dirptr = NULL;
 
@@ -1123,9 +1073,7 @@ static int dotconf_handle_star(const command_t *cmd, const char *path,
 	s_ext = ext;
 
 	while (dotconf_is_wild_card(*s_ext)) /* remove trailing wild-cards proceeded by * */
-	{
 		s_ext++;
-	}
 
 	t_ext = s_ext;
 
@@ -1155,24 +1103,14 @@ static int dotconf_handle_star(const command_t *cmd, const char *path,
 				if ( !alloced )
 				{
 					if((new_path = malloc(new_path_len)) == NULL)
-					{
 						return -1;
-					}
 
 					alloced = new_path_len;
 
-				} else {
-
-						if ( new_path_len > alloced )
-						{
-							if ( realloc(new_path,new_path_len) == NULL )
-							{
-								free(new_path);
-								return -1;
-							}
-
-						}
-
+                                } else if(new_path_len > alloced &&
+                                  realloc(new_path,new_path_len) == NULL) {
+                                        free(new_path);
+                                        return -1;
 				}
 
 				if (match_state == 1)
@@ -1190,9 +1128,7 @@ static int dotconf_handle_star(const command_t *cmd, const char *path,
 					}
 
 					if (sub_count + t_ext_count > name_len)
-					{
 						continue;
-					}
 
 					strncpy(new_pre, dirptr->d_name, sub_count + t_ext_count);
 					new_pre[sub_count+t_ext_count] = '\0';
@@ -1201,15 +1137,10 @@ static int dotconf_handle_star(const command_t *cmd, const char *path,
 					sprintf(new_path,"%s%s%s",path,new_pre,t_ext);
 
 					if (strcmp(new_path,already_matched) == 0)
-					{
 						continue; /* Already searched this expression */
-
-					} else {
-
+					else
 						strcpy(already_matched,new_path);
 
-					}
-
 					if (dotconf_find_wild_card(new_path,&wc,&wc_path,&wc_pre,&wc_ext) >= 0)
 					{
 						if ( dotconf_handle_wild_card(cmd,wc,wc_path,wc_pre,wc_ext) < 0)
@@ -1264,7 +1195,7 @@ static int dotconf_handle_star(const command_t *cmd, const char *path,
 static DOTCONF_CB(dotconf_cb_include) {
 	char *filename = NULL, *path = NULL, *pre = NULL;
         const char *ext = NULL;
-	configfile_t *included;
+	struct configfile *included;
 	char wild_card;
 
 	if (cmd->configfile->includepath != NULL
@@ -1295,7 +1226,7 @@ static DOTCONF_CB(dotconf_cb_include) {
 				 cmd->configfile->includepath, sl, cmd->data.str);
 	}
 	else						/* fully qualified, or no includepath */
-		filename = strdup(cmd->data.str);
+		filename = xstrdup(cmd->data.str);
 
 	/* Added wild card support here */
 	if (dotconf_find_wild_card(filename,&wild_card,&path,&pre,&ext) >= 0)
