@@ -1,7 +1,7 @@
 /*=============================================================================
 fmt_ptrn.c
   Copyright (C) W. Michael Putello <new@flyn.org>, 1999 - 2000
-  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2005
+  Copyright © Jan Engelhardt <jengelh [at] gmx de>, 2005 - 2006
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
@@ -28,53 +28,63 @@ fmt_ptrn.c
 #include <string.h>
 #include <zlib.h>
 #include "buffer.h"
+#include "compiler.h"
 #include "modifiers.h"
 #include "fmt_ptrn.h"
+#include "pair.h"
+#include "xstdlib.h"
+
+// Definitions
 #define KEY_LEN         80
 #define PARSE_ERR_LEN   BUFSIZ
 #define STACK_MAX_ITEMS 10
 
-typedef struct mystack_t {
-	modifier_t data[STACK_MAX_ITEMS];
-	int size;
-} mystack_t;
+struct mystack {
+    struct modifier data[STACK_MAX_ITEMS];
+    int size;
+};
 
-static void     _apply_modifiers(fmt_ptrn_t *, buffer_t *, mystack_t *);
+// Functions
+static void     _apply_modifiers(struct fmt_ptrn *, struct buffer *, struct mystack *);
 static gint     _cmp(gconstpointer, gconstpointer);
-static gboolean _copy_fillers(gpointer, gpointer, gpointer);
-static void     _eat_alternate(fmt_ptrn_t *, const char **);
-static gboolean _fill_it(fmt_ptrn_t *, const char *);
-static int      _fmt_ptrn_copy_fillers(fmt_ptrn_t *, fmt_ptrn_t *);
-static gboolean _fmt_ptrn_t_valid(const fmt_ptrn_t *);
-static gboolean _free_tree_node(gpointer, gpointer, gpointer);
-static void     _handle_fmt_str(fmt_ptrn_t *, const char **);
-static gboolean _is_literal(fmt_ptrn_t *, const char *);
-static gboolean _lookup(const fmt_ptrn_t *, const char *, buffer_t *);
+static bool     _copy_fillers(gpointer, gpointer, gpointer);
+static void     _eat_alternate(struct fmt_ptrn *, const char **);
+static bool     _fill_it(struct fmt_ptrn *, const char *);
+static int      _fmt_ptrn_copy_fillers(struct fmt_ptrn *, struct fmt_ptrn *);
+static int      fmt_ptrn_valid(const struct fmt_ptrn *);
+static bool     _free_tree_node(gpointer, gpointer, gpointer);
+static void     _handle_fmt_str(struct fmt_ptrn *, const char **);
+static bool     _is_literal(struct fmt_ptrn *, const char *);
+static bool     _lookup(const struct fmt_ptrn *, const char *, struct buffer *);
 static char *   _matching_paren(const char *);
-static inline gboolean _modifier_t_valid(const modifier_t *);
-static void     _read_alternate(fmt_ptrn_t *, const char **, buffer_t *);
-static void     _read_key(fmt_ptrn_t *, char *, const char **);
-static void     _read_literal(fmt_ptrn_t *, char *, buffer_t *);
-static gboolean _read_modifier(fmt_ptrn_t *, const char **, mystack_t *);
-static void     _read_modifier_arg(fmt_ptrn_t *, const char **, modifier_t *);
-static void     _read_modifiers(fmt_ptrn_t *, const char **, mystack_t *);
-static int      _stack_contains(const mystack_t, const char *);
-static inline void _stack_init(mystack_t *);
-static gboolean _stack_pop(mystack_t *, modifier_t *);
-static gboolean _stack_push(fmt_ptrn_t *, mystack_t *, const modifier_t *);
-static inline gboolean _stack_t_valid(const mystack_t *);
+static inline bool modifier_valid(const struct modifier *);
+static void     _read_alternate(struct fmt_ptrn *, const char **, struct buffer *);
+static void     _read_key(struct fmt_ptrn *, char *, const char **);
+static void     _read_literal(struct fmt_ptrn *, char *, struct buffer *);
+static bool     _read_modifier(struct fmt_ptrn *, const char **, struct mystack *);
+static void     _read_modifier_arg(struct fmt_ptrn *, const char **, struct modifier *);
+static void     _read_modifiers(struct fmt_ptrn *, const char **, struct mystack *);
+static int      _stack_contains(const struct mystack, const char *);
+static inline void _stack_init(struct mystack *);
+static bool     _stack_pop(struct mystack *, struct modifier *);
+static bool     _stack_push(struct fmt_ptrn *, struct mystack *, const struct modifier *);
+static inline bool mystack_valid(const struct mystack *);
 
-/* ============================ _fmt_ptrn_t_valid () ======================= */
-static gboolean _fmt_ptrn_t_valid(const fmt_ptrn_t * x)
-{
+//-----------------------------------------------------------------------------
+/*  fmt_ptrn_valid
+    @x: fmt_ptrn to check
+
+    Verifies that the fmt_ptrn strucutre is consistent.
+*/
+static int fmt_ptrn_valid(const struct fmt_ptrn *x) {
 	if (x == NULL)
 		return FALSE;
 	/* FIXME */
 	/* gzFile template_fp; */
 	/* char template_path[PATH_MAX + 1]; */
 	/* long line_num; */
-        if(!buffer_t_valid(&x->raw_buf) || !buffer_t_valid(&x->filled_buf) ||
-         !buffer_t_valid(&x->lookup_buf))
+        if(!buffer_valid(&x->raw_buf) || !buffer_valid(&x->filled_buf) ||
+         !buffer_valid(&x->lookup_buf))
 		return FALSE;
 	/* char errmsg[BUFSIZ + 1]; */
 	/* GQueue *parse_errmsg; */
@@ -83,92 +93,72 @@ static gboolean _fmt_ptrn_t_valid(const fmt_ptrn_t * x)
 }
 
 /* ============================ fmt_ptrn_parse_err () ====================== */
-gboolean fmt_ptrn_parse_err(const fmt_ptrn_t * x)
-{
-	assert(_fmt_ptrn_t_valid(x));
+bool fmt_ptrn_parse_err(const struct fmt_ptrn *x) {
+	assert(fmt_ptrn_valid(x));
 	return !g_queue_is_empty(x->parse_errmsg);
 }
 
 /* ============================ fmt_ptrn_parse_strerror () ================= */
-char *fmt_ptrn_parse_strerror(fmt_ptrn_t * x)
-{
+char *fmt_ptrn_parse_strerror(struct fmt_ptrn *x) {
 	char *errmsg;
 	char *fnval;
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 
 	if((errmsg = g_queue_pop_tail(x->parse_errmsg)) == NULL)
-		fnval = g_strdup("no error"); // leak
+		fnval = xstrdup("no error"); // leak
 	else
 		fnval = errmsg; // g_queue_pop_tail: possible leak
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(fnval != NULL);
 
 	return fnval;
 }
 
-/* ============================ fmt_ptrn_parse_perror () =================== */
-void fmt_ptrn_parse_perror(fmt_ptrn_t * x, const char *msg)
-{
-	char *errmsg;
-
-	assert(_fmt_ptrn_t_valid(x));
-
-	errmsg = fmt_ptrn_parse_strerror(x);
-	if(msg != NULL)
-		fprintf(stderr, "%s: %s\n", msg, errmsg);
-	else
-		fprintf(stderr, "%s\n", errmsg);
-	g_free(errmsg);
-
-	assert(_fmt_ptrn_t_valid(x));
-}
-
 /* ============================ enqueue_parse_errmsg () =================== */
-void enqueue_parse_errmsg(fmt_ptrn_t * x, const char *msg, ...)
-{
+void enqueue_parse_errmsg(struct fmt_ptrn *x, const char *msg, ...) {
 	va_list args;
 	char *err;
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(msg != NULL);
 
-	err = g_new0(char, PARSE_ERR_LEN + 1);
+	err = xmalloc(PARSE_ERR_LEN + 1);
 	va_start(args, msg);
 	vsnprintf(err, PARSE_ERR_LEN, msg, args);
 	va_end(args);
 	g_queue_push_head(x->parse_errmsg, err);
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 }
 
-/* ============================ _modifier_t_valid () ======================= */
-static inline gboolean _modifier_t_valid(const modifier_t * m) {
+/* ============================ modifier_valid () ======================= */
+static inline bool modifier_valid(const struct modifier *m) {
 	/* FIXME */
 	return TRUE;
 }
 
-/* ============================ _stack_t_valid () ========================== */
-static inline gboolean _stack_t_valid(const mystack_t *s) {
+/* ============================ mystack_valid () ========================== */
+static inline bool mystack_valid(const struct mystack *s) {
     return (s == NULL && s->size != 0) ? FALSE : TRUE;
 }
 
 /* ============================ _stack_init () ============================= */
-static inline void _stack_init(mystack_t *s) {
+static inline void _stack_init(struct mystack *s) {
 	s->size = 0;
-	assert(_stack_t_valid(s));
+	assert(mystack_valid(s));
 }
 
 /* ============================ _stack_push () ============================= */
-static gboolean _stack_push(fmt_ptrn_t *x, mystack_t *s,
- const modifier_t *data)
+static bool _stack_push(struct fmt_ptrn *x, struct mystack *s,
+ const struct modifier *data)
 {
-	gboolean fnval = FALSE;
+	bool fnval = FALSE;
 
-	assert(_fmt_ptrn_t_valid(x));
-	assert(_stack_t_valid(s));
-	assert(_modifier_t_valid(data));
+	assert(fmt_ptrn_valid(x));
+	assert(mystack_valid(s));
+	assert(modifier_valid(data));
 	if (s->size == STACK_MAX_ITEMS) {
 		enqueue_parse_errmsg(x, "%s: %ld: more than %d modifiers",
 				     x->template_path, x->line_num,
@@ -179,17 +169,16 @@ static gboolean _stack_push(fmt_ptrn_t *x, mystack_t *s,
 		fnval = TRUE;
 	}
 
-	assert(_fmt_ptrn_t_valid(x));
-	assert(_stack_t_valid(s));
+	assert(fmt_ptrn_valid(x));
+	assert(mystack_valid(s));
 	return fnval;
 }
 
 /* ============================ _stack_pop () ============================== */
-static gboolean _stack_pop(mystack_t *s, modifier_t *data)
-{
-	gboolean fnval = FALSE;
-	assert(_stack_t_valid(s));
-	assert(_modifier_t_valid(data));
+static bool _stack_pop(struct mystack *s, struct modifier *data) {
+	bool fnval = FALSE;
+	assert(mystack_valid(s));
+	assert(modifier_valid(data));
 	if(s->size == 0)
 		fnval = FALSE;
 	else {
@@ -197,42 +186,31 @@ static gboolean _stack_pop(mystack_t *s, modifier_t *data)
 		fnval = TRUE;
 	}
 
-	assert(_stack_t_valid(s));
-	assert(_modifier_t_valid(data));
+	assert(mystack_valid(s));
+	assert(modifier_valid(data));
 	return fnval;
 }
 
 /* ============================ _stack_contains () ========================= */
-static int _stack_contains(const mystack_t s, const char *n)
+static int _stack_contains(const struct mystack s, const char *n)
 {
 	int i;
-	assert(_stack_t_valid(&s));
+	assert(mystack_valid(&s));
 	for (i = 0; i < s.size; i++)
 		if (s.data[i].fn.id == n)
 			return 1;
 	return 0;
 }
 
-/* ============================ fmt_ptrn_update_kv_p () ==================== */
-void fmt_ptrn_update_kv_p(fmt_ptrn_t * x, const pair_t * p)
-{
-	assert(_fmt_ptrn_t_valid(x));
-	assert(pair_t_valid(p));
-	/* [MP] FIXME: this strdups here but other fn requires malloced strs!
-           [JE] No problem here, strdup uses malloc. */
-	g_tree_insert(x->fillers, strdup(p->key), strdup(p->val));
-	assert(_fmt_ptrn_t_valid(x));
-}
-
 /* ============================ fmt_ptrn_update_kv () ====================== */
-void fmt_ptrn_update_kv(fmt_ptrn_t * x, const char *key, const char *val)
-{
-	assert(_fmt_ptrn_t_valid(x));
+void fmt_ptrn_update_kv(struct fmt_ptrn *x, const char *key, const char *val) {
+	assert(fmt_ptrn_valid(x));
 	assert(key != NULL);
 	assert(val != NULL);
 	/* FIXME: getting rid of the const is silly, but I didn't write g_tree_insert */
-	g_tree_insert(x->fillers, (char *) key, (char *) val);
-	assert(_fmt_ptrn_t_valid(x));
+        // string not duplicated but freed? see above
+	g_tree_insert(x->fillers, const_cast(char *, key), const_cast(char *, val));
+	assert(fmt_ptrn_valid(x));
 }
 
 /* ============================ _matching_paren () ========================= */
@@ -248,57 +226,59 @@ static char *_matching_paren(const char *str) {
 		else if (*str == ')')
 			count--;
 		if (count == 0)
-			return (char *)str;
+			return const_cast(char *, str);
 		str++;
 	}
 	return NULL;
 }
 
 /* ============================ _copy_fillers () =========================== */
-static gboolean _copy_fillers(gpointer key, gpointer val, gpointer data) {
+static bool _copy_fillers(gpointer key, gpointer val, gpointer data) {
 	assert(key != NULL);
 	assert(val != NULL);
-	assert(_fmt_ptrn_t_valid(data));
+	assert(fmt_ptrn_valid(data));
 
-	g_tree_insert(((fmt_ptrn_t *) data)->fillers,
-		      strdup(key), strdup(val));
+	g_tree_insert(static_cast(struct fmt_ptrn *, data)->fillers,
+		      xstrdup(key), xstrdup(val));
 
-	assert(_fmt_ptrn_t_valid(data));
+	assert(fmt_ptrn_valid(data));
 
 	return FALSE;
 }
 
 /* ============================ _fmt_ptrn_copy_fillers () ================== */
-static int _fmt_ptrn_copy_fillers(fmt_ptrn_t *x, fmt_ptrn_t *y) {
+static int _fmt_ptrn_copy_fillers(struct fmt_ptrn *x, struct fmt_ptrn *y) {
 /* Copies fillers from one fmt_ptrn to another. */
-	assert(_fmt_ptrn_t_valid(x));
-	assert(_fmt_ptrn_t_valid(y));
+	assert(fmt_ptrn_valid(x));
+	assert(fmt_ptrn_valid(y));
 
 	/* FIXME: tried using g_node_copy but that did not seem to work */
 	g_tree_foreach(y->fillers, _copy_fillers, x);
 
-	assert(_fmt_ptrn_t_valid(x));
-	assert(_fmt_ptrn_t_valid(y));
+	assert(fmt_ptrn_valid(x));
+	assert(fmt_ptrn_valid(y));
 
 	return 1;
 }
 
 /* ============================ _read_alternate () ========================= */
-static void _read_alternate(fmt_ptrn_t * x, const char **p, buffer_t *buf) {
+static void _read_alternate(struct fmt_ptrn *x, const char **p,
+ struct buffer *buf)
+{
 	char *alt_end;
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(p != NULL);
 	assert(*p != NULL);
-	assert(buffer_t_valid(buf));
+	assert(buffer_valid(buf));
 	if(**p == '\0')		/* Already queued error, hopefully. */
 		return;
 	if (**p == ':') {
-		(*p)++;
+		++*p;
 		if((alt_end = _matching_paren(*p)) != NULL) {
 			/* FIXME: clean up? */
-			fmt_ptrn_t y;
+			struct fmt_ptrn y;
 			char *alt, *filled_alt;
-			alt = g_strndup(*p, (alt_end - *p));
+			alt = xstrndup(*p, (alt_end - *p));
 			fmt_ptrn_init(&y);
 			_fmt_ptrn_copy_fillers(&y, x);
 			filled_alt = fmt_ptrn_filled(&y, alt);
@@ -308,8 +288,8 @@ static void _read_alternate(fmt_ptrn_t * x, const char **p, buffer_t *buf) {
 						     fmt_ptrn_parse_strerror
 						     (&y));
 			*p += (alt_end - *p);
-			g_free(alt);
-			g_free(filled_alt);
+			free(alt);
+			free(filled_alt);
 			fmt_ptrn_close(&y);
 		} else
 			enqueue_parse_errmsg(x, "%s: %ld: end of input",
@@ -322,16 +302,16 @@ static void _read_alternate(fmt_ptrn_t * x, const char **p, buffer_t *buf) {
 		 */
 		enqueue_parse_errmsg(x, "%s: %ld: key has no value",
 				     x->template_path, x->line_num);
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(p != NULL);
 	assert(*p != NULL);
-	assert(buffer_t_valid(buf));
+	assert(buffer_valid(buf));
 }
 
 /* ============================ _eat_alternate () ========================== */
-static void _eat_alternate(fmt_ptrn_t *x, const char **pattern) {
+static void _eat_alternate(struct fmt_ptrn *x, const char **pattern) {
 	char *alt_end;
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(pattern != NULL);
 	assert(*pattern != NULL);
 	if(**pattern == '\0' || **pattern != ':')
@@ -342,22 +322,22 @@ static void _eat_alternate(fmt_ptrn_t *x, const char **pattern) {
 	if(**pattern == '\0')
 		enqueue_parse_errmsg(x, "%s: %ld: end of input",
 				     x->template_path, x->line_num);
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(pattern != NULL);
 	assert(*pattern != NULL);
 }
 
 /* ============================ _read_modifier_arg () ====================== */
-static void _read_modifier_arg(fmt_ptrn_t *x, const char **pattern,
- modifier_t *i)
+static void _read_modifier_arg(struct fmt_ptrn *x, const char **pattern,
+ struct modifier *i)
 {
 	size_t arg_len;
 	char *end_quote, *end_paren;
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(pattern != NULL);
 	assert(*pattern != NULL);
-	assert(_modifier_t_valid(i));
+	assert(modifier_valid(i));
 
 	end_quote = strchr(*pattern, '"');
 	end_paren = strchr(*pattern, ')');
@@ -372,7 +352,7 @@ static void _read_modifier_arg(fmt_ptrn_t *x, const char **pattern,
 					     "%s: %ld: modifier arg. too long",
 					     x->template_path,
 					     x->line_num);
-		} else 
+		} else
 			g_strlcpy(i->arg, *pattern, arg_len + 1);
 		if (*(*pattern + arg_len + 1) != ' ')
 			enqueue_parse_errmsg(x,
@@ -382,24 +362,24 @@ static void _read_modifier_arg(fmt_ptrn_t *x, const char **pattern,
 		*pattern += arg_len + 2;	/* Add 2 for end quote and space. */
 	}
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(pattern != NULL);
 	assert(*pattern != NULL);
-	assert(_modifier_t_valid(i));
+	assert(modifier_valid(i));
 }
 
 /* ============================ _read_modifier () ========================== */
-static gboolean _read_modifier(fmt_ptrn_t *x, const char **ptrn,
- mystack_t *modifier)
+static bool _read_modifier(struct fmt_ptrn *x, const char **ptrn,
+ struct mystack *modifier)
 {
 	int i = 0;
-	modifier_t m;
-	gboolean fnval = FALSE;
+	struct modifier m;
+	bool fnval = FALSE;
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(ptrn != NULL);
 	assert(*ptrn != NULL);
-	assert(_stack_t_valid(modifier));
+	assert(mystack_valid(modifier));
 
 	while(mod_fn[i].id != NULL) {
 		if(strncmp(mod_fn[i].id, *ptrn, strlen(mod_fn[i].id)) == 0) {
@@ -418,40 +398,38 @@ static gboolean _read_modifier(fmt_ptrn_t *x, const char **ptrn,
 		i++;
 	}
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(ptrn != NULL);
 	assert(*ptrn != NULL);
-	assert(_stack_t_valid(modifier));
+	assert(mystack_valid(modifier));
 
 	return fnval;
 }
 
 /* ============================ _read_modifiers () ========================= */
-static void _read_modifiers(fmt_ptrn_t *x, const char **ptrn,
- mystack_t *modifier)
+static void _read_modifiers(struct fmt_ptrn *x, const char **ptrn,
+ struct mystack *modifier)
 {
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(ptrn != NULL);
 	assert(*ptrn != NULL);
-	assert(_stack_t_valid(modifier));
+	assert(mystack_valid(modifier));
 
-	while (_read_modifier(x, ptrn, modifier)) {
-		/* NOOP. */
-	}
+	while(_read_modifier(x, ptrn, modifier)) /* noop */;
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(ptrn != NULL);
 	assert(*ptrn != NULL);
-	assert(_stack_t_valid(modifier));
+	assert(mystack_valid(modifier));
 
 	return;
 }
 
 /* ============================ _read_key () =============================== */
-static void _read_key(fmt_ptrn_t *x, char *key, const char **p) {
+static void _read_key(struct fmt_ptrn *x, char *key, const char **p) {
 	int i;
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(key != NULL);
 	assert(p != NULL);
 	assert(*p != NULL);
@@ -462,7 +440,7 @@ static void _read_key(fmt_ptrn_t *x, char *key, const char **p) {
 	if(**p != '\0' && strchr(":)", **p) == NULL) {
 		/* Uh oh, key is too many characters, eat the rest. */
 		while(**p != '\0' && **p != ':' && **p != ')')
-			(*p)++;
+			++*p;
 		enqueue_parse_errmsg(x, "%s: %ld: key too long",
 				     x->template_path, x->line_num);
 	}
@@ -470,48 +448,42 @@ static void _read_key(fmt_ptrn_t *x, char *key, const char **p) {
 		enqueue_parse_errmsg(x, "%s: %ld: end of input",
 				     x->template_path, x->line_num);
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(p != NULL);
 	assert(*p != NULL);
 }
 
 /* ============================ _apply_modifiers () ======================== */
-static void _apply_modifiers(fmt_ptrn_t * x,
-			     buffer_t *str, mystack_t *modifier)
+static void _apply_modifiers(struct fmt_ptrn *x, struct buffer *str,
+ struct mystack *modifier)
 {
-	modifier_t m;
+	struct modifier m;
 
-	assert(_fmt_ptrn_t_valid(x));
-	assert(buffer_t_valid(str));
-	assert(_stack_t_valid(modifier));
+	assert(fmt_ptrn_valid(x));
+	assert(buffer_valid(str));
+	assert(mystack_valid(modifier));
 
 	if (buffer_len(str) > 0)	/* error should have been queued elsewhere */
-		while (_stack_pop(modifier, &m))
-			if(m.fn.fn != NULL)
-				if (!m.fn.fn(str, x, m.arg))
-					enqueue_parse_errmsg(x,
-							     "%s: %ld: error applying %s modifier to %s",
-							     x->
-							     template_path,
-							     x->line_num,
-							     m.fn.id,
-							     str->data);
+            while(_stack_pop(modifier, &m))
+                if(m.fn.fn != NULL && !m.fn.fn(str, x, m.arg))
+                    enqueue_parse_errmsg(x, "%s: %ld: error applying %s modifier to %s",
+                      x->template_path, x->line_num, m.fn.id, str->data);
 
-	assert(_fmt_ptrn_t_valid(x));
-	assert(buffer_t_valid(str));
-	assert(_stack_t_valid(modifier));
+	assert(fmt_ptrn_valid(x));
+	assert(buffer_valid(str));
+	assert(mystack_valid(modifier));
 }
 
 /* ============================ _lookup () ================================= */
-static gboolean _lookup(const fmt_ptrn_t *x, const char *key,
- buffer_t *value)
+static bool _lookup(const struct fmt_ptrn *x, const char *key,
+ struct buffer *value)
 {
 	char *tmp;
-	gboolean fnval = FALSE;
+	bool fnval = FALSE;
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(key != NULL);
-	assert(buffer_t_valid(value));
+	assert(buffer_valid(value));
 
 	tmp = g_tree_lookup(x->fillers, key);
 	if(tmp != NULL) {
@@ -523,16 +495,16 @@ static gboolean _lookup(const fmt_ptrn_t *x, const char *key,
 		fnval = FALSE;
 	}
 
-	assert(buffer_t_valid(value));
+	assert(buffer_valid(value));
 
 	return fnval;
 }
 
 /* ============================ _is_literal () ============================= */
-static gboolean _is_literal(fmt_ptrn_t *x, const char *str) {
-	gboolean fnval = FALSE;
+static bool _is_literal(struct fmt_ptrn *x, const char *str) {
+	bool fnval = FALSE;
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(str != NULL);
 
 	if (*str == '"') {
@@ -543,18 +515,17 @@ static gboolean _is_literal(fmt_ptrn_t *x, const char *str) {
 		fnval = TRUE;
 	}
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 
 	return fnval;
 }
 
 /* ============================ _read_literal () =========================== */
 /* FIXME: is this right?  it does not seem to look for closing '"' */
-static void _read_literal(fmt_ptrn_t *x, char *str, buffer_t * buf)
-{
-	assert(_fmt_ptrn_t_valid(x));
+static void _read_literal(struct fmt_ptrn *x, char *str, struct buffer *buf) {
+	assert(fmt_ptrn_valid(x));
 	assert(str != NULL);
-	assert(buffer_t_valid(buf));
+	assert(buffer_valid(buf));
 
 	str++;
 	str[strlen(str) - 1] = '\0';
@@ -565,17 +536,17 @@ static void _read_literal(fmt_ptrn_t *x, char *str, buffer_t * buf)
 					     x->template_path,
 					     x->line_num);
 
-	assert(_fmt_ptrn_t_valid(x));
-	assert(buffer_t_valid(buf));
+	assert(fmt_ptrn_valid(x));
+	assert(buffer_valid(buf));
 }
 
 /* ============================ _handle_fmt_str () ========================= */
-static void _handle_fmt_str(fmt_ptrn_t *x, const char **p) {
+static void _handle_fmt_str(struct fmt_ptrn *x, const char **p) {
 	/* format string -> %(<modifier_0> ... <modifier_n> <key>:<alt>) */
-	mystack_t modifier;
+	struct mystack modifier;
 	char key[KEY_LEN + 1];
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(p != NULL);
 	assert(*p != NULL);
 
@@ -599,48 +570,48 @@ static void _handle_fmt_str(fmt_ptrn_t *x, const char **p) {
 			realloc_n_cat(&x->filled_buf, x->lookup_buf.data);
 	}
 	if(**p != '\0')
-		(*p)++;		/* Skip ')'. */
+		++*p; /* Skip ')'. */
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(p != NULL);
 	assert(*p != NULL);
 }
 
 /* ============================ _fill_it () ================================ */
-static gboolean _fill_it(fmt_ptrn_t *x, const char *p) {
-	const char *pattern, *orig_ptr;
-	gboolean fnval = TRUE;
+static bool _fill_it(struct fmt_ptrn *x, const char *p) {
+	const char *pattern;
+        char *orig_ptr;
+	bool fnval = TRUE;
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(p != NULL);
 
-	pattern = orig_ptr = g_strdup(p);
+	pattern = orig_ptr = xstrdup(p);
 	while(*pattern != '\0') {
 		if (*pattern == '%' && *(pattern + 1) == '%') {
 			/* Handle %%(...), which should be filled as %(...). */
 			realloc_n_ncat(&x->filled_buf, pattern, 1);
 			pattern += 2;
-		} else if (*pattern == '%' && *(pattern + 1) == '(')
+		} else if(*pattern == '%' && *(pattern + 1) == '(') {
 			_handle_fmt_str(x, &pattern);
-		else {
+		} else {
 			if (*pattern == '\n')
 				x->line_num++;
 			realloc_n_ncat(&x->filled_buf, pattern++, 1);
 		}
 	}
-	g_free((char *)orig_ptr);
+	free(orig_ptr);
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 
 	return fnval;
 }
 
 /* ============================ fmt_ptrn_filled () ========================= */
-char *fmt_ptrn_filled(fmt_ptrn_t * x, const char *p)
-{
+char *fmt_ptrn_filled(struct fmt_ptrn *x, const char *p) {
 	char *fnval = NULL;
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	assert(p != NULL);
 
 	buffer_clear(&x->filled_buf);
@@ -648,9 +619,9 @@ char *fmt_ptrn_filled(fmt_ptrn_t * x, const char *p)
 		return NULL;
 	if(buffer_len(&x->filled_buf) > 0)
 		/* FIXME: what if len == 0? protected by assert, but... */
-		fnval = g_strdup(x->filled_buf.data);
+		fnval = xstrdup(x->filled_buf.data);
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 	/* FIXME: assert(fnval != NULL); WHY DID I THINK THIS WAS NEEDED? */
 
 	return fnval;
@@ -669,11 +640,10 @@ static gint _cmp(gconstpointer a, gconstpointer b) {
 }
 
 /* ============================ fmt_ptrn_init () =========================== */
-int fmt_ptrn_init(fmt_ptrn_t * x)
-/* Alternative to open; does everything but open the file.  This 
+int fmt_ptrn_init(struct fmt_ptrn *x) {
+/* Alternative to open; does everything but open the file.  This
  * should be used when filling strings instead of files.
  */
-{
 	g_strlcpy(x->errmsg, "no error", sizeof(x->errmsg));
 	x->parse_errmsg = g_queue_new();
 	x->fillers = g_tree_new(_cmp);
@@ -683,25 +653,24 @@ int fmt_ptrn_init(fmt_ptrn_t * x)
         buffer_init(&x->filled_buf);
         buffer_init(&x->lookup_buf);
 	g_strlcpy(x->template_path, "string", sizeof(x->template_path));
-	
-	assert(_fmt_ptrn_t_valid(x));
+
+	assert(fmt_ptrn_valid(x));
 
 	return 1;
 }
 
 /* ============================ fmt_ptrn_open () =========================== */
-gboolean fmt_ptrn_open(const char *path, fmt_ptrn_t * x)
-{
+bool fmt_ptrn_open(const char *path, struct fmt_ptrn *x) {
 	gzFile in_file;
-	gboolean fnval = TRUE;
+	bool fnval = TRUE;
 
 	assert(path != NULL);
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 
 	if((in_file = gzopen(path, "rb")) == NULL) {
 		fnval = FALSE;
 		goto _return;
-	} 
+	}
 	if (!fmt_ptrn_init(x)) {
 		fnval = FALSE;
 		goto _return;
@@ -709,22 +678,21 @@ gboolean fmt_ptrn_open(const char *path, fmt_ptrn_t * x)
 	x->template_fp = in_file;	/* init sets this to NULL. */
 	strcpy(x->template_path, path);	/* init sets this to "string". */
 _return:
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 
 	return fnval;
 }
 
 /* ============================ fmt_ptrn_gets () =========================== */
-char *fmt_ptrn_gets(char *buf, size_t size, fmt_ptrn_t * x)
-{
+char *fmt_ptrn_gets(char *buf, size_t size, struct fmt_ptrn *x) {
 	char *fnval = NULL;
 
 	assert(buf != NULL);
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 
 	if (buffer_len(&x->filled_buf) == 0) {
-		/* FIXME: potentially, a buffer could be filled with only 
-		 * half of a format string. 
+		/* FIXME: potentially, a buffer could be filled with only
+		 * half of a format string.
 		 */
 		/* Here buf is used as a temp. buffer. */
 		if (gzgets(x->template_fp, buf, size) == Z_NULL) {
@@ -738,56 +706,42 @@ char *fmt_ptrn_gets(char *buf, size_t size, fmt_ptrn_t * x)
 	}
 	if (buffer_len(&x->filled_buf) > 0) {
 		g_strlcpy(buf, x->filled_buf.data, size);
-		buffer_eat(x->filled_buf, strlen(buf));
+		buffer_eat(&x->filled_buf, strlen(buf));
 		fnval = buf;
 	} else {
 		fnval = NULL;
 		goto _return;
 	}
 _return:
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 
 	return fnval;
 }
 
 /* ============================ _free_tree_node () ========================= */
 /* FIXME: this function should take TWO pointers!!!!!! */
-static gboolean _free_tree_node(gpointer key, gpointer val, gpointer data) {
+static bool _free_tree_node(gpointer key, gpointer val, gpointer data) {
 /* FIXME: this function may not modify tree.  need to write pointers to a list and then destroy that list outside of this function. */
 	return FALSE;
 }
 
 /* ============================ fmt_ptrn_close () ========================== */
-int fmt_ptrn_close(fmt_ptrn_t * x)
-{
+int fmt_ptrn_close(struct fmt_ptrn *x) {
 	gpointer ptr;
 
-	assert(_fmt_ptrn_t_valid(x));
+	assert(fmt_ptrn_valid(x));
 
 	while ((ptr = g_queue_pop_head(x->parse_errmsg)) != NULL)
-		g_free(ptr);
+		free(ptr);
 	g_tree_foreach(x->fillers, _free_tree_node, NULL);
 	buffer_clear(&x->raw_buf);
 	buffer_clear(&x->filled_buf);
 	buffer_clear(&x->lookup_buf);
-	/* x->template_fp == NULL if fmt_ptrn_init was used instead of 
+	/* x->template_fp == NULL if fmt_ptrn_init was used instead of
 	 * fmt_ptrn_open.
 	 */
 	return (x != NULL && x->template_fp != NULL) ?
             gzclose(x->template_fp) : 1;
 }
 
-/* ============================ fmt_ptrn_perror () ========================= */
-void fmt_ptrn_perror(const fmt_ptrn_t *x, const char *msg) {
-	assert(_fmt_ptrn_t_valid(x));
-	assert(msg != NULL);
-
-	fprintf(stderr, "%s: %s\n", msg, x->errmsg);
-}
-
-/* ============================ fmt_ptrn_strerror () ======================= */
-const char *fmt_ptrn_strerror(const fmt_ptrn_t *x) {
-	assert(_fmt_ptrn_t_valid(x));
-
-	return x->errmsg;
-}
+//=============================================================================
