@@ -1,7 +1,7 @@
 /*=============================================================================
 mount.c
   Copyright (C) Elvis Pfützenreuter <epx@conectiva.com>, 2000
-  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2005
+  Copyright © Jan Engelhardt <jengelh [at] gmx de>, 2005 - 2006
   Copyright © Bastian Kleineidam <calvin [at] debian org>, 2005
 
   This program is free software; you can redistribute it and/or modify
@@ -36,6 +36,7 @@ mount.c
 #include <string.h>
 #include <unistd.h>
 #include <pwd.h>
+#include "compiler.h"
 #include "crypto.h"
 #include "fmt_ptrn.h"
 #include "misc.h"
@@ -44,7 +45,8 @@ mount.c
 #include "pam_mount.h"
 #include "private.h"
 #include "readconfig.h"
-#include "xprot.h"
+#include "spawn.h"
+#include "xstdlib.h"
 #if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
 #    include <fstab.h>
 #elif defined(__linux__)
@@ -62,51 +64,53 @@ mount.c
 #    define EVP_MAX_BLOCK_LENGTH 0 // FIXME: this is ugly, but needed
 #endif
 
-static int already_mounted(const config_t * const, const unsigned int,
-    char * const, fmt_ptrn_t *);
-static int check_filesystem(const config_t *, const unsigned int, fmt_ptrn_t *,
-    const unsigned char *, size_t);
-static int do_losetup(const config_t *, const unsigned int, fmt_ptrn_t *,
-    const unsigned char *, size_t);
-static int do_unlosetup(const config_t *, fmt_ptrn_t *);
+// Functions
+static int already_mounted(const struct config * const, const unsigned int, char * const, struct fmt_ptrn *);
+static int check_filesystem(const struct config *, const unsigned int, struct fmt_ptrn *, const unsigned char *, size_t);
+static int do_losetup(const struct config *, const unsigned int, struct fmt_ptrn *, const unsigned char *, size_t);
+static int do_unlosetup(const struct config *, struct fmt_ptrn *);
 static int fstype_nodev(const char *);
 static void log_output(int);
-static void log_pm_input(const config_t * const, const unsigned int);
+static void log_pm_input(const struct config * const, const unsigned int);
 static inline const char *loop_bk(const char *, struct loop_info64 *);
-static int mkmountpoint(vol_t * const, const char * const);
+static int mkmountpoint(struct vol * const, const char * const);
 static int pipewrite(int, const void *, size_t);
-static void run_lsof(const config_t * const, fmt_ptrn_t *);
-static void vol_to_dev(char *, size_t, const vol_t *);
+static void run_lsof(const struct config * const, struct fmt_ptrn *);
+static void vol_to_dev(char *, size_t, const struct vol *);
 
 #if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
 static int split_bsd_mount(char *, const char **, const char **, const char **);
 #endif
 
-/* ============================ log_output () ============================== */
-/* INPUT: fd, a valid file descriptor
- * SIDE EFFECTS: contents of fd are logged, usually fd is connected by a 
- *               pipe to another process's stdout or stderr
- */
-static void log_output(int fd)
-{
+//-----------------------------------------------------------------------------
+/*  log_output
+    @fd:        file descriptor to read from
+
+    Reads all data from @fd and logs it using w4rn(). @fd is uaually connected
+    to a pipe to a nother process's stdout or stderr.
+*/
+static void log_output(int fd) {
 	FILE *fp;
 	char buf[BUFSIZ + 1];
-	/* FIXME: check fdopen's retval */
 	if ((fp = fdopen(fd, "r")) == NULL) {
-		w4rn(PMPREFIX "error opening file: %s\n",
-		     strerror(errno));
+		w4rn(PMPREFIX "error opening file: %s\n", strerror(errno));
 		return;
 	}
 	while(fgets(buf, sizeof(buf), fp) != NULL)
-		w4rn(PMPREFIX "%s\n", buf);
+		w4rn(PMPREFIX "%s", buf);
 }
 
-/* ============================ run_lsof () ================================ */
-/*
- * NOTE: this fn simply runs lsof on a directory and logs its output for
- * debugging purposes
- */
-static void run_lsof(const config_t *const config, fmt_ptrn_t *vinfo) {
+
+/*  run_lsof
+    @config:    current configuration
+    @vinfo:
+
+    Runs `lsof` on a directory/mountpoint and logs its output, for debugging
+    purposes.
+*/
+static void run_lsof(const struct config *const config,
+ struct fmt_ptrn *vinfo)
+{
 	int i, _argc = 0, cstdout = -1, child_exit;
 	const char *_argv[MAX_PAR + 1];
 	GError *err = NULL;
@@ -115,10 +119,10 @@ static void run_lsof(const config_t *const config, fmt_ptrn_t *vinfo) {
 		l0g(PMPREFIX "lsof not defined in pam_mount.conf\n");
 	/* FIXME: NEW */
 	for(i = 0; config->command[i][LSOF] != NULL; i++)
-		add_to_argv(_argv, &_argc, config->command[i][LSOF],
-			    vinfo);
+		add_to_argv(_argv, &_argc, config->command[i][LSOF], vinfo);
 	log_argv(_argv);
-        if(!spawn_ap0(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
+
+        if(!spawn_apS(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
 	  &pid, NULL, &cstdout, NULL, &err)) {
 		l0g(PMPREFIX "%s\n", err->message);
 		g_error_free(err);
@@ -128,34 +132,34 @@ static void run_lsof(const config_t *const config, fmt_ptrn_t *vinfo) {
 	log_output(cstdout);
 	w4rn(PMPREFIX "waiting for lsof\n");
 	if (waitpid(pid, &child_exit, 0) == -1)
-		l0g(PMPREFIX "error waiting for child\n");
+		l0g(PMPREFIX "error waiting for child: %s\n", strerror(errno));
+        spawn_restore_sigchld();
 	CLOSE(cstdout);
 }
 
-/* ============================ already_mounted () ========================= */
-/*
- * PRE:    config->volume[vol].type is a mount type (LCLMOUNT, SMBMOUNT, ...)
- *         sizeof(mntpt) >= PATH_MAX + 1
- *         vinfo is a valid struct fmt_ptrn_t
- * POST:   mntpt contains:
- *           config->volume[vol].mountpoint if config->volume[vol].volume is
- *             already mounted there
- *           some other path if config->volume[vol].volume is mounted elsewhere
- *           undefined if config->volume[vol].volume is not mounted
- * FN VAL: 1 if config->volume[vol].volume is mounted, 0 if not, -1 on error
- *         errors are logged
- */
-static int already_mounted(const config_t *const config,
- const unsigned int vol, char *const mntpt, fmt_ptrn_t *vinfo)
+
+/*  already_mounted
+    @config:    current config
+    @vol:       volume index into @config->volume[]
+    @mntpt:     destination buffer for current mountpoint
+    @vinfo:
+
+    Checks if @config->volume[@vol] is already mounted, and if so, writes the
+    mountpoint into @mntpt (which must be at least of size %PATH_MAX+1) and
+    returns 1. If the volume is not mounted, returns zero and @mntpt is
+    cleared. Returns -1 on error.
+*/
+static int already_mounted(const struct config *const config,
+ const unsigned int vol, char *const mntpt, struct fmt_ptrn *vinfo)
 #if defined(__linux__)
 {
     char dev[PATH_MAX+1] = {}, real_mpt[PATH_MAX+1];
     struct mntent *mtab_record;
     int mounted = 0;
     FILE *mtab;
-    vol_t *vpt;
- 
-    assert(config_t_valid(config));
+    struct vol *vpt;
+
+    assert(config_valid(config));
     vpt = &config->volume[vol];
     vol_to_dev(dev, sizeof(dev), vpt);
 
@@ -165,16 +169,18 @@ static int already_mounted(const config_t *const config,
     }
     if(realpath(vpt->mountpoint, real_mpt) == NULL) {
         w4rn(PMPREFIX "can't get realpath of volume %s: %s\n",
-          vpt->mountpoint, strerror(errno));
+             vpt->mountpoint, strerror(errno));
         strncpy(real_mpt, vpt->mountpoint, PATH_MAX);
         real_mpt[PATH_MAX] = '\0';
     } else {
         real_mpt[PATH_MAX] = '\0';
         l0g(PMPREFIX "realpath of volume \"%s\" is \"%s\"\n",
-          vpt->mountpoint, real_mpt);
+            vpt->mountpoint, real_mpt);
     }
+
+    *mntpt = '\0';
     w4rn(PMPREFIX "checking to see if %s is already mounted at %s\n",
-      dev, vpt->mountpoint);
+         dev, vpt->mountpoint);
 
     while((mtab_record = getmntent(mtab)) != NULL) {
         const char *fsname = mtab_record->mnt_fsname;
@@ -185,11 +191,10 @@ static int already_mounted(const config_t *const config,
         struct stat statbuf;
 
         if(stat(fsname, &statbuf) == 0 && S_ISBLK(statbuf.st_mode) &&
-         major(statbuf.st_rdev) == LOOP_MAJOR) {
+         major(statbuf.st_rdev) == LOOP_MAJOR)
             /* If /etc/mtab is a link to /proc/mounts then the loop device
             instead of the real device will be listed -- resolve it. */
             fsname = loop_bk(fsname, &loopdev);
-        }
 
         xcmp = (strcmp(fstype, "smbfs") == 0 || strcmp(fstype, "cifs") == 0 ||
                 strcmp(fstype, "ncpfs") == 0) ? strcasecmp : strcmp;
@@ -211,11 +216,11 @@ static int already_mounted(const config_t *const config,
     char *_argv[MAX_PAR + 1], dev[PATH_MAX+1] = {}, mte[BUFSIZ + 1];
     int i, _argc = 0, cstdout = -1, mounted = 0;
     GError *err = NULL;
-    vol_t *vpt;
+    struct vol *vpt;
     pid_t pid;
     FILE *fp;
 
-    assert(config_t_valid(config));
+    assert(config_valid(config));
     vpt = &config->volume[vol];
     vol_to_dev(dev, sizeof(dev), vpt);
 
@@ -226,13 +231,13 @@ static int already_mounted(const config_t *const config,
         return -1;
     }
 
-    for(i = 0; config->command[i][MNTCHECK] != NULL; ++i) {
+    for(i = 0; config->command[i][MNTCHECK] != NULL; ++i)
         add_to_argv(_argv, &_argc, config->command[i][MNTCHECK], vinfo);
-    }
+
     log_argv(_argv);
 
     // FIXME: replace by popen() if available on BSD
-    if(!spawn_ap0(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
+    if(!spawn_apS(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
      &pid, NULL, &cstdout, NULL, &err)) {
         l0g(PMPREFIX "%s\n", err->message);
         g_error_free(err);
@@ -245,7 +250,7 @@ static int already_mounted(const config_t *const config,
         int (*xcmp)(const char *, const char *);
         const char *fsname, *fstype, *fspt;
 
-        w4rn(PMPREFIX "mounted filesystem: %s", mte); // MTE includes '\n'
+        w4rn(PMPREFIX "mounted filesystem: %s", mte); // @mte includes '\n'
         if(!split_bsd_mount(mte, &fsname, &fspt, &fstype)) {
             mounted = -1;
             break;
@@ -253,7 +258,7 @@ static int already_mounted(const config_t *const config,
 
         // Use case-insensitive for SMB, etc.
         // FIXME: Is it called "smbfs" under BSD too?
-        xcmp = (fstype != NULL && *fstype != '\0' &&
+        xcmp = (fstype != NULL &&
                (strcmp(fstype, "smbfs") == 0 || strcmp(fstype, "cifs") == 0 ||
                strcmp(fstype, "ncpfs") == 0)) ? strcasecmp : strcmp;
 
@@ -268,7 +273,9 @@ static int already_mounted(const config_t *const config,
     }
 
     fclose(fp); // automatically closes cstdout, too
-    waitpid(pid, NULL, 0);
+    if(waitpid(pid, NULL, 0) != 0)
+        l0g(PMPREFIX "error waiting for child: %s\n", strerror(errno));
+    spawn_restore_sigchld();
     return mounted;
 }
 #else
@@ -279,7 +286,14 @@ static int already_mounted(const config_t *const config,
 }
 #endif
 
-static void vol_to_dev(char *match, size_t s, const vol_t *vol) {
+
+/*  vol_to_dev
+    @match:
+    @s:
+    @vol:       volume to analyze
+
+*/
+static void vol_to_dev(char *match, size_t s, const struct vol *vol) {
     switch(vol->type) {
         case SMBMOUNT:
         case CIFSMOUNT:
@@ -345,10 +359,10 @@ static int split_bsd_mount(char *wp, const char **fsname, const char **fspt,
 #endif
 
 /* ============================ log_pm_input () ============================ */
-static void log_pm_input(const config_t *const config,
+static void log_pm_input(const struct config *const config,
  const unsigned int vol)
 {
-        const vol_t *vpt = &config->volume[vol];
+        const struct vol *vpt = &config->volume[vol];
 	char options[MAX_PAR + 1];
 
 	w4rn(PMPREFIX "information for mount:\n");
@@ -372,12 +386,12 @@ static void log_pm_input(const config_t *const config,
 /* POST:   the directory named d exists && volume->created_mntpt = TRUE
  * FN VAL: if error 0 else 1, errors are logged
  */
-static int mkmountpoint(vol_t *const volume, const char *const d) {
+static int mkmountpoint(struct vol *const volume, const char *const d) {
 	int ret = 1;
 	struct passwd *passwd_ent;
 	char dcopy[PATH_MAX + 1], *parent;
 
-	assert(vol_t_valid(volume));
+	assert(vol_valid(volume));
 	assert(d != NULL);
 
 	w4rn(PMPREFIX "creating mount point %s\n", d);
@@ -395,8 +409,7 @@ static int mkmountpoint(vol_t *const volume, const char *const d) {
 			ret = 0;
 			goto _return;
 		}
-		if (chown(d, passwd_ent->pw_uid, passwd_ent->pw_gid)
-		    != 0) {
+		if(chown(d, passwd_ent->pw_uid, passwd_ent->pw_gid) != 0) {
 			l0g(PMPREFIX "could not chown %s to %s\n",
 			    d, volume->user);
 			ret = 0;
@@ -409,28 +422,28 @@ static int mkmountpoint(vol_t *const volume, const char *const d) {
 	}
 	volume->created_mntpt = TRUE;
       _return:
-	g_free(parent);
+	free(parent);
 	return ret;
 }
 
-int do_unmount(const config_t *config, const unsigned int vol,
- fmt_ptrn_t *vinfo, const char *const password, const gboolean mkmntpoint)
-/* PRE:    config points to a valid config_t*
- *         config->volume[vol] is a valid struct vol_t
- *         vinfo is a valid struct fmt_ptrn_t
+int do_unmount(const struct config *config, const unsigned int vol,
+ struct fmt_ptrn *vinfo, const char *const password, const bool mkmntpoint)
+{
+/* PRE:    config points to a valid struct config
+ *         config->volume[vol] is a valid struct vol
+ *         vinfo is a valid struct fmt_ptrn
  *         mkmntpoint is true if mount point should be rmdir'ed
  * POST:   volume is unmounted
  * FN VAL: if error 0 else 1, errors are logged
  */
-{
 	GError *err = NULL;
 	int i, child_exit, _argc = 0, ret = 1, cstderr = -1;
 	pid_t pid = -1;
 	const char *_argv[MAX_PAR + 1];
-        const vol_t *vpt;
+        const struct vol *vpt;
         int type;
 
-	assert(config_t_valid(config));
+	assert(config_valid(config));
 	assert(vinfo != NULL);
 	assert(password == NULL);	/* password should point to NULL for unmounting */
 
@@ -446,6 +459,7 @@ int do_unmount(const config_t *config, const unsigned int vol,
         switch(vpt->type) {
             case SMBMOUNT: type = SMBUMOUNT; break;
             case NCPMOUNT: type = NCPUMOUNT; break;
+            case FUSEMOUNT: type = FUSEUMOUNT; break;
             default:       type = UMOUNT; break;
         }
 
@@ -459,11 +473,11 @@ int do_unmount(const config_t *config, const unsigned int vol,
 	 * util-linux will have native dm_crypt support some day */
 	if(vpt->type == CRYPTMOUNT) {
 		_argc = 0;
-		add_to_argv(_argv, &_argc, "/usr/bin/umount.crypt", vinfo);
+		add_to_argv(_argv, &_argc, "/sbin/umount.crypt", vinfo);
 		add_to_argv(_argv, &_argc, "%(MNTPT)", vinfo);
 	}
 	log_argv(_argv);
-        if(!spawn_ap0(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, set_myuid,
+        if(!spawn_apS(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, set_myuid,
           NULL, &pid, NULL, NULL, &cstderr, &err)) {
 		l0g(PMPREFIX "%s\n", err->message);
 		g_error_free(err);
@@ -475,7 +489,7 @@ int do_unmount(const config_t *config, const unsigned int vol,
 	CLOSE(cstderr);
 	w4rn(PMPREFIX "waiting for umount\n");
 	if (waitpid(pid, &child_exit, 0) == -1) {
-		l0g(PMPREFIX "error waiting for child\n");
+		l0g(PMPREFIX "error waiting for child: %s\n", strerror(errno));
 		ret = 0;
 		goto _return;
 	} else {
@@ -483,6 +497,7 @@ int do_unmount(const config_t *config, const unsigned int vol,
 		ret = !WEXITSTATUS(child_exit);
 	}
       _return:
+        spawn_restore_sigchld();
         if(mkmntpoint != 0 && vpt->created_mntpt &&
           rmdir(vpt->mountpoint) == -1) /* non-fatal */
 		w4rn(PMPREFIX "could not remove %s\n", vpt->mountpoint);
@@ -518,24 +533,24 @@ _return:
 	return fnval;
 }
 
-static int do_losetup(const config_t *config, const unsigned int vol,
- fmt_ptrn_t *vinfo, const unsigned char *password, size_t password_len)
-/* PRE:    config points to a valid config_t*
- *         config->volume[vol] is a valid struct vol_t
- *         vinfo is a valid struct fmt_ptrn_t
+static int do_losetup(const struct config *config, const unsigned int vol,
+ struct fmt_ptrn *vinfo, const unsigned char *password, size_t password_len)
+{
+/* PRE:    config points to a valid struct config
+ *         config->volume[vol] is a valid struct vol
+ *         vinfo is a valid struct fmt_ptrn
  *         config->volume[vol].options is valid
  * POST:   volume has associated with a loopback device
  * FN VAL: if error 0 else 1, errors are logged
  */
-{
 	pid_t pid;
 	GError *err = NULL;
 	int i, ret = 1, child_exit, _argc = 0, cstdin = -1, cstderr = -1;
 	const char *_argv[MAX_PAR + 1];
         const char *cipher, *keybits;
-        const vol_t *vpt;
+        const struct vol *vpt;
 
-	assert(config_t_valid(config));
+	assert(config_valid(config));
 	assert(vinfo != NULL);
 	assert(password != NULL);
 	/* password_len is unsigned */
@@ -556,12 +571,11 @@ static int do_losetup(const config_t *config, const unsigned int vol,
 		if(keybits != NULL)
 			fmt_ptrn_update_kv(vinfo, "KEYBITS", keybits);
 	}
-	for(i = 0; config->command[i][LOSETUP] != NULL; i++) {
-		add_to_argv(_argv, &_argc,
-			    config->command[i][LOSETUP], vinfo);
-	}
+	for(i = 0; config->command[i][LOSETUP] != NULL; ++i)
+            add_to_argv(_argv, &_argc, config->command[i][LOSETUP], vinfo);
+
 	log_argv(_argv);
-        if(!spawn_ap0(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, set_myuid,
+        if(!spawn_apS(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, set_myuid,
           NULL, &pid, &cstdin, NULL, &cstderr, &err)) {
 		l0g(PMPREFIX "%s\n", err->message);
 		g_error_free(err);
@@ -580,18 +594,19 @@ static int do_losetup(const config_t *config, const unsigned int vol,
 	CLOSE(cstderr);
 	w4rn(PMPREFIX "waiting for losetup\n");
 	if (waitpid(pid, &child_exit, 0) == -1) {
-		l0g(PMPREFIX "error waiting for child\n");
+		l0g(PMPREFIX "error waiting for child: %s\n", strerror(errno));
 		ret = 0;
 	} else if(ret > 0) {
 		/* pass on through the result from the losetup process */
 		ret = !WEXITSTATUS(child_exit);
 	}
+        spawn_restore_sigchld();
 	return ret;
 }
 
-static int do_unlosetup(const config_t *config, fmt_ptrn_t *vinfo) {
-/* PRE:    config points to a valid config_t*
- *         vinfo is a valid struct fmt_ptrn_t
+static int do_unlosetup(const struct config *config, struct fmt_ptrn *vinfo) {
+/* PRE:    config points to a valid struct config
+ *         vinfo is a valid struct fmt_ptrn
  * POST:   volume has associated with a loopback device
  * FN VAL: if error 0 else 1, errors are logged
  */
@@ -600,7 +615,7 @@ static int do_unlosetup(const config_t *config, fmt_ptrn_t *vinfo) {
 	const char *_argv[MAX_PAR + 1];
 	int i, child_exit, _argc = 0;
 
-	assert(config_t_valid(config));
+	assert(config_valid(config));
 	assert(vinfo != NULL);
 
 	if(config->command[0][UNLOSETUP] == NULL) {
@@ -613,27 +628,29 @@ static int do_unlosetup(const config_t *config, fmt_ptrn_t *vinfo) {
 		add_to_argv(_argv, &_argc,
 			    config->command[i][UNLOSETUP], vinfo);
 	log_argv(_argv);
-        if(!spawn_ap0(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
+        if(!spawn_apS(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
           &pid, NULL, NULL, NULL, &err)) {
 		l0g(PMPREFIX "%s\n", err->message);
 		g_error_free(err);
 		return 0;
 	}
 	w4rn(PMPREFIX "waiting for losetup delete\n");
-	waitpid(pid, &child_exit, 0);
+        if(waitpid(pid, &child_exit, 0) != 0)
+            l0g(PMPREFIX "error waiting for child: %s\n", strerror(errno));
+        spawn_restore_sigchld();
 	/* pass on through the result */
 	return !WEXITSTATUS(child_exit);
 }
 
-static int check_filesystem(const config_t *config, const unsigned int vol,
- fmt_ptrn_t * vinfo, const unsigned char *password, size_t password_len)
-/* PRE:    config points to a valid struct config_t*
- *         config->volume[vol] is a valid struct vol_t
- *         vinfo is a valid struct fmt_ptrn_t
+static int check_filesystem(const struct config *config, const unsigned int vol,
+ struct fmt_ptrn *vinfo, const unsigned char *password, size_t password_len)
+{
+/* PRE:    config points to a valid struct config
+ *         config->volume[vol] is a valid struct vol
+ *         vinfo is a valid struct fmt_ptrn
  * POST:   integrity of volume has been checked
  * FN VAL: if error 0 else 1, errors are logged
  */
-{
 #if defined (__linux__)
 	pid_t pid;
 	GError *err = NULL;
@@ -641,9 +658,9 @@ static int check_filesystem(const config_t *config, const unsigned int vol,
 	const char *_argv[MAX_PAR + 1];
         char options[MAX_PAR + 1];
         const char *fsck_target;
-        const vol_t *vpt;
+        const struct vol *vpt;
 
-	assert(config_t_valid(config));
+	assert(config_valid(config));
 	assert(vinfo != NULL);
 	assert(password != NULL);
 	assert(password_len >= 0
@@ -660,11 +677,10 @@ static int check_filesystem(const config_t *config, const unsigned int vol,
         if(optlist_exists(vpt->options, "bind") ||
          optlist_exists(vpt->options, "move") ||
          fstype_nodev(vpt->fstype) != 0)
-            return 1;
+                return 1;
 
 	if (optlist_exists(vpt->options, "loop")) {
-		if (!do_losetup
-		    (config, vol, vinfo, password, password_len))
+		if(!do_losetup(config, vol, vinfo, password, password_len))
 			return 0;
 		fsck_target = config->fsckloop;
 	} else
@@ -674,10 +690,10 @@ static int check_filesystem(const config_t *config, const unsigned int vol,
 	/* FIXME: need to fsck /dev/mapper/whatever... */
 	fmt_ptrn_update_kv(vinfo, "FSCKTARGET", fsck_target);
 	for (i = 0; config->command[i][FSCK]; i++)
-		add_to_argv(_argv, &_argc, config->command[i][FSCK],
-			    vinfo);
+            add_to_argv(_argv, &_argc, config->command[i][FSCK], vinfo);
+
 	log_argv(_argv);
-        if(!spawn_ap0(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
+        if(!spawn_apS(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
           &pid, NULL, &cstdout, &cstderr, &err)) {
 		l0g(PMPREFIX "%s\n", err->message);
 		g_error_free(err);
@@ -687,30 +703,33 @@ static int check_filesystem(const config_t *config, const unsigned int vol,
 	log_output(cstderr);
 	CLOSE(cstderr);
 	w4rn(PMPREFIX "waiting for filesystem check\n");
-	waitpid(pid, &child_exit, 0);
+        if(waitpid(pid, &child_exit, 0) != 0)
+            l0g(PMPREFIX "error waiting for child: %s\n", strerror(errno));
+        spawn_restore_sigchld();
 	if(optlist_exists(vpt->options, "loop"))
 		if (!do_unlosetup(config, vinfo))
 			return 0;
-	/* pass on through the result -- okay if 0 (no errors) 
+	/* pass on through the result -- okay if 0 (no errors)
 	 * or 1 (errors corrected) */
-	return (WEXITSTATUS(child_exit) == 0
-		|| WEXITSTATUS(child_exit) == 1);
+	return WEXITSTATUS(child_exit) == 0 || WEXITSTATUS(child_exit) == 1;
 #else
 	l0g(PMPREFIX "checking filesystem not implemented on arch.\n");
 	return 1;
 #endif
 }
 
-int do_mount(const config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
- const char *password, const gboolean mkmntpoint)
-/* PRE:    config points to a valid struct config_t*
- *         config->volume[vol] is a valid struct vol_t
- *         vinfo is a valid struct fmt_ptrn_t
- *         mkmntpoint is true if mount point should be mkdir'ed
- *         false if mount point was received from pam_mount
- * POST:   volume is mounted
- * FN VAL: if error 0 else 1, errors are logged
- */
+
+/*  do_mount
+    @config:    current config
+    @vol:       volume index into @config->vol[]
+    @vinfo:     valid struct fmt_ptrn
+    @password:
+    @mkmntpoint: whether to create mountpoint if it does not exist
+
+    Returns zero on error, positive non-zero for success.
+*/
+int do_mount(const struct config *config, const unsigned int vol,
+ struct fmt_ptrn *vinfo, const char *password, const bool mkmntpoint)
 {
 	const char *_argv[MAX_PAR + 1];
 	char prev_mntpt[PATH_MAX + 1];
@@ -719,9 +738,9 @@ int do_mount(const config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
 	unsigned char _password[MAX_PAR + EVP_MAX_BLOCK_LENGTH];
 	int _argc = 0, child_exit = 0, cstdin = -1, cstderr = -1;
 	pid_t pid = -1;
-        vol_t *vpt;
+        struct vol *vpt;
 
-	assert(config_t_valid(config));
+	assert(config_valid(config));
 	assert(vinfo != NULL);
 	assert(password != NULL);
 
@@ -764,7 +783,7 @@ int do_mount(const config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
 			add_to_argv(_argv, &_argc,
 				    config->command[i][MNTAGAIN], vinfo);
 		log_argv(_argv);
-                if(!spawn_ap0(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
+                if(!spawn_apS(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
                   set_myuid, NULL, &pid, NULL, NULL, &cstderr, &err)) {
 			l0g(PMPREFIX "%s\n", err->message);
 			g_error_free(err);
@@ -772,6 +791,7 @@ int do_mount(const config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
 		}
 	} else {
 		GError *err = NULL;
+                char *mount_user;
 		if(config->command[0][vpt->type] == NULL) {
 			l0g(PMPREFIX "proper mount command not defined in pam_mount.conf\n");
 			return 0;
@@ -790,9 +810,9 @@ int do_mount(const config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
 				return 0;
 		} else {
 			/* _password is an ASCII string in this case -- we'll treat its
-			 * MAX_PAR + EVP_MAX_BLOCK_LENGTH size as the standard string 
+			 * MAX_PAR + EVP_MAX_BLOCK_LENGTH size as the standard string
 			 * MAX_PAR + 1 in this case */
-			strncpy((char *) _password, password, MAX_PAR);
+			strncpy(signed_cast(char *, _password), password, MAX_PAR);
 			_password[MAX_PAR] = '\0';
 			_password_len = strlen(password);
 		}
@@ -802,9 +822,9 @@ int do_mount(const config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
 		   l0g(PMPREFIX "volume type (%d) is unknown\n", vpt->type);
 		   return 0;
 		 */
-                for(i = 0; config->command[i][vpt->type] != NULL; ++i) {
+                for(i = 0; config->command[i][vpt->type] != NULL; ++i)
                     add_to_argv(_argv, &_argc, config->command[i][vpt->type], vinfo);
-                }
+
                 if(vpt->type == LCLMOUNT &&
                   !check_filesystem(config, vol, vinfo, _password, _password_len))
 			l0g(PMPREFIX "error checking filesystem but will continue\n");
@@ -812,15 +832,15 @@ int do_mount(const config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
                 if(vpt->type == SMBMOUNT || vpt->type == CIFSMOUNT)
 			setenv("PASSWD_FD", "0", 1);
 		log_argv(_argv);
-                if(!spawn_ap0(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
-                  set_myuid, NULL, &pid, &cstdin, NULL, &cstderr, &err)) {
+                mount_user = strcmp(vpt->fstype, "fuse") == 0 ? vpt->user : NULL;
+                if(!spawn_apS(NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
+                  set_myuid, mount_user, &pid, &cstdin, NULL, &cstderr, &err)) {
 			l0g(PMPREFIX "%s\n", err->message);
 			g_error_free(err);
 			return 0;
 		}
                 if(vpt->type != NFSMOUNT) {
-			if (pipewrite(cstdin, _password, _password_len) !=
-			    _password_len)
+			if(pipewrite(cstdin, _password, _password_len) != _password_len)
 				/* FIXME: clean: returns value of exit below */
 				l0g(PMPREFIX "error sending password to mount\n");
 		}
@@ -833,10 +853,12 @@ int do_mount(const config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
 	CLOSE(cstderr);
 	w4rn(PMPREFIX "waiting for mount\n");
 	if (waitpid(pid, &child_exit, 0) == -1) {
-		l0g(PMPREFIX "error waiting for child\n");
+                spawn_restore_sigchld();
+		l0g(PMPREFIX "error waiting for child: %s\n", strerror(errno));
 		return 0;
 	}
 
+        spawn_restore_sigchld();
         if(Debug)
             system("df");
 
@@ -844,30 +866,27 @@ int do_mount(const config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
         return !WEXITSTATUS(child_exit);
 }
 
-/* ============================ mount_op () ================================ */
-/*
- * PRE:    mnt is function to execute mount operations: do_mount or do_unmount
- *         vol > 0
- *         config points to a valid struct config_t
- *         config->volume[vol] is a valid struct vol_t
- *         password points to a valid string (can be NULL if UNmounting)
- *         mkmntpoint is true if mount point should be created when it does
- *         not already exist
- * POST:   appropriate mount or unmount operations are performed
- * FN VAL: if error 0 else 1, errors are logged
- * NOTE:   * checked by volume_record_sane
- *         ** checked by read_volume()
- */
-int mount_op(int (*mnt)(const config_t *, const unsigned int, fmt_ptrn_t *,
- const char *, const int), const config_t *config, const unsigned int vol,
- const char *password, const int mkmntpoint)
+
+/*  mount_op
+    @mnt:       function to execute mount operations (do_mount or do_unmount)
+    @config:
+    @vol:       volume index into @config->volume[]
+    @password:  password string (may be %NULL on unmount)
+    @mkmntpoint: whether to create mountpoint if it does not exist
+
+    Returns zero on error, positive non-zero for success.
+    Note: Checked by volume_record_sane() and read_volume()
+*/
+int mount_op(mount_op_fn_t *mnt, const struct config *config,
+ const unsigned int vol, const char *password, const int mkmntpoint)
 {
 	int fnval;
-	fmt_ptrn_t vinfo;
-	char options[MAX_PAR + 1];
-        const vol_t *vpt;
+	struct fmt_ptrn vinfo;
+	char options[MAX_PAR + 1], uuid[16], ugid[16];
+        const struct vol *vpt;
+        struct passwd *pe;
 
-	assert(config_t_valid(config));
+	assert(config_valid(config));
 
         vpt = &config->volume[vol];
 
@@ -878,6 +897,16 @@ int mount_op(int (*mnt)(const config_t *, const unsigned int, fmt_ptrn_t *,
 	fmt_ptrn_update_kv(&vinfo, "VOLUME", vpt->volume);
 	fmt_ptrn_update_kv(&vinfo, "SERVER", vpt->server);
 	fmt_ptrn_update_kv(&vinfo, "USER", vpt->user);
+        if((pe = getpwnam(vpt->user)) == NULL) {
+            w4rn(PMPREFIX "getpwnam(\"%s\") failed: %s\n",
+             Config.user, strerror(errno));
+        } else {
+            snprintf(uuid, sizeof(uuid), "%ld", static_cast(long, pe->pw_uid));
+            snprintf(ugid, sizeof(ugid), "%ld", static_cast(long, pe->pw_gid));
+            fmt_ptrn_update_kv(&vinfo, "USERUID", uuid);
+            fmt_ptrn_update_kv(&vinfo, "USERGID", ugid);
+        }
+
 	/* FIXME: should others remain undefined if == ""? */
 	optlist_to_str(options, vpt->options);
 	fmt_ptrn_update_kv(&vinfo, "OPTIONS", options);
@@ -890,6 +919,7 @@ int mount_op(int (*mnt)(const config_t *, const unsigned int, fmt_ptrn_t *,
 	return fnval;
 }
 
+
 /* copied from libHX */
 /* noproto */ static
 char *HX_chomp(char *s) {
@@ -903,11 +933,14 @@ char *HX_chomp(char *s) {
     return s;
 }
 
-static int fstype_nodev(const char *name) {
-    /* Returns 1 if the filesystem does not require a block device,
-    0 if it does require a block device,
-    -1 if we could not find out. */
 
+/*  fstype_nodev
+    @name:      fstype to check
+
+    Returns 1 if the filesystem does not require a block device, 0 if it does
+    require a block device, -1 if we could not find out.
+*/
+static int fstype_nodev(const char *name) {
     char buf[MAX_PAR];
     FILE *fp;
 
@@ -929,19 +962,36 @@ static int fstype_nodev(const char *name) {
     return -1;
 }
 
+
+/*  loop_bk
+    @filename:  block device to query
+    @i:         pointer to result storage
+
+    Run the LOOP_GET_STATUS64 ioctl on @filename and store the result in @i.
+    Returns the underlying file of the loop device, or @filename if @filename
+    does not seem to be a loop device at all.
+*/
 static inline const char *loop_bk(const char *filename,
  struct loop_info64 *i)
 {
     int fd;
-    if((fd = open(filename, O_RDONLY)) < 0) {
+    if((fd = open(filename, O_RDONLY)) < 0)
         return filename;
-    }
+
+#ifndef LOOP_GET_STATUS64
+#    error -------------------------------------------------------------------
+#    error Your userspace kernel headers (/usr/include/linux/) are out of date
+#    error This is NOT a pam-mount bug, but one of your distribution.
+#    error See www.sourceforge.net/mailarchive/message.php?msg_id=13974728
+#    error for details.
+#    error -------------------------------------------------------------------
+#endif
     if(ioctl(fd, LOOP_GET_STATUS64, i) != 0) {
         close(fd);
         return filename;
     }
     close(fd);
-    return (char *)i->lo_file_name; // signed_cast<>
+    return signed_cast(char *, i->lo_file_name);
 }
 
 //=============================================================================
