@@ -1,6 +1,6 @@
 /*=============================================================================
 pam_mount.h
-  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2005
+  Copyright © Jan Engelhardt <jengelh [at] gmx de>, 2005 - 2006
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
@@ -22,16 +22,34 @@ pam_mount.h
 #ifndef PMT_PAM_MOUNT_H
 #define PMT_PAM_MOUNT_H 1
 
+#include <glib.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#include <glib.h>
-#include "private.h"
+struct config;
+struct pam_args;
 
+enum auth_type {
+    GET_PASS,
+    USE_FIRST_PASS,
+    TRY_FIRST_PASS
+};
+
+struct pam_args {
+    enum auth_type auth_type;
+};
+
+/*
+ *      PAM_MOUNT.C
+ */
+#if defined(__GNUC__) && __GNUC__ < 4
+#    define Debug pmt_debug
+#endif
 extern int Debug;
-extern config_t Config;
-extern pam_args_t Args;
+extern struct config Config;
+extern struct pam_args Args;
 
 #ifdef __cplusplus
 } // extern "C"
